from yattag import Doc

from ..exceptions import WQXException
from .BibliographicReference import BibliographicReference
from .MeasureCompact import MeasureCompact
from .SimpleContent import (
    CommentText,
    LocationCategoryName,
    LocationStatusName,
    MonitoringLocationIdentifier,
    ReferenceLocationEndDate,
    ReferenceLocationStartDate,
    ReferenceLocationTypeCode,
    StatisticalStratumText,
)


class ProjectMonitoringLocationWeighting:
    """
    Describes the probability weighting information for a given Project / Monitoring
    Location Assignment.
    """

    __monitoringLocationIdentifier: MonitoringLocationIdentifier
    __locationWeightingFactorMeasure: MeasureCompact
    __statisticalStratumText: StatisticalStratumText
    __locationCategoryName: LocationCategoryName
    __locationStatusName: LocationStatusName
    __referenceLocationTypeCode: ReferenceLocationTypeCode
    __referenceLocationStartDate: ReferenceLocationStartDate
    __referenceLocationEndDate: ReferenceLocationEndDate
    __referenceLocationCitation: BibliographicReference
    __commentText: CommentText

    def __init__(
        self,
        o: dict = None,
        *,
        monitoringLocationIdentifier: MonitoringLocationIdentifier = None,
        locationWeightingFactorMeasure: MeasureCompact = None,
        statisticalStratumText: StatisticalStratumText = None,
        locationCategoryName: LocationCategoryName = None,
        locationStatusName: LocationStatusName = None,
        referenceLocationTypeCode: ReferenceLocationTypeCode = None,
        referenceLocationStartDate: ReferenceLocationStartDate = None,
        referenceLocationEndDate: ReferenceLocationEndDate = None,
        referenceLocationCitation: BibliographicReference = None,
        commentText: CommentText = None
    ):
        if isinstance(o, ProjectMonitoringLocationWeighting):
            # Assign attributes from object without typechecking
            self.__monitoringLocationIdentifier = o.monitoringLocationIdentifier
            self.__locationWeightingFactorMeasure = o.locationWeightingFactorMeasure
            self.__statisticalStratumText = o.statisticalStratumText
            self.__locationCategoryName = o.locationCategoryName
            self.__locationStatusName = o.locationStatusName
            self.__referenceLocationTypeCode = o.referenceLocationTypeCode
            self.__referenceLocationStartDate = o.referenceLocationStartDate
            self.__referenceLocationEndDate = o.referenceLocationEndDate
            self.__referenceLocationCitation = o.referenceLocationCitation
            self.__commentText = o.commentText
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.monitoringLocationIdentifier = o.get("monitoringLocationIdentifier")
            self.locationWeightingFactorMeasure = o.get("locationWeightingFactorMeasure")
            self.statisticalStratumText = o.get("statisticalStratumText")
            self.locationCategoryName = o.get("locationCategoryName")
            self.locationStatusName = o.get("locationStatusName")
            self.referenceLocationTypeCode = o.get("referenceLocationTypeCode")
            self.referenceLocationStartDate = o.get("referenceLocationStartDate")
            self.referenceLocationEndDate = o.get("referenceLocationEndDate")
            self.referenceLocationCitation = o.get("referenceLocationCitation")
            self.commentText = o.get("commentText")
        else:
            # Assign attributes from named keywords with typechecking
            self.monitoringLocationIdentifier = monitoringLocationIdentifier
            self.locationWeightingFactorMeasure = locationWeightingFactorMeasure
            self.statisticalStratumText = statisticalStratumText
            self.locationCategoryName = locationCategoryName
            self.locationStatusName = locationStatusName
            self.referenceLocationTypeCode = referenceLocationTypeCode
            self.referenceLocationStartDate = referenceLocationStartDate
            self.referenceLocationEndDate = referenceLocationEndDate
            self.referenceLocationCitation = referenceLocationCitation
            self.commentText = commentText

    @property
    def monitoringLocationIdentifier(self) -> MonitoringLocationIdentifier:
        return self.__monitoringLocationIdentifier

    @monitoringLocationIdentifier.setter
    def monitoringLocationIdentifier(self, val: MonitoringLocationIdentifier) -> None:
        self.__monitoringLocationIdentifier = MonitoringLocationIdentifier(val)

    @property
    def locationWeightingFactorMeasure(self) -> MeasureCompact:
        """
        A measurement of the monitoring location selection weighting factor.
        """
        return self.__locationWeightingFactorMeasure

    @locationWeightingFactorMeasure.setter
    def locationWeightingFactorMeasure(self, val: MeasureCompact) -> None:
        """
        A measurement of the monitoring location selection weighting factor.
        """
        self.__locationWeightingFactorMeasure = MeasureCompact(val)

    @property
    def statisticalStratumText(self) -> StatisticalStratumText:
        return self.__statisticalStratumText

    @statisticalStratumText.setter
    def statisticalStratumText(self, val: StatisticalStratumText) -> None:
        self.__statisticalStratumText = (
            None if val is None else StatisticalStratumText(val)
        )

    @property
    def locationCategoryName(self) -> LocationCategoryName:
        return self.__locationCategoryName

    @locationCategoryName.setter
    def locationCategoryName(self, val: LocationCategoryName) -> None:
        self.__locationCategoryName = None if val is None else LocationCategoryName(val)

    @property
    def locationStatusName(self) -> LocationStatusName:
        return self.__locationStatusName

    @locationStatusName.setter
    def locationStatusName(self, val: LocationStatusName) -> None:
        self.__locationStatusName = None if val is None else LocationStatusName(val)

    @property
    def referenceLocationTypeCode(self) -> ReferenceLocationTypeCode:
        return self.__referenceLocationTypeCode

    @referenceLocationTypeCode.setter
    def referenceLocationTypeCode(self, val: ReferenceLocationTypeCode) -> None:
        self.__referenceLocationTypeCode = (
            None if val is None else ReferenceLocationTypeCode(val)
        )

    @property
    def referenceLocationStartDate(self) -> ReferenceLocationStartDate:
        return self.__referenceLocationStartDate

    @referenceLocationStartDate.setter
    def referenceLocationStartDate(self, val: ReferenceLocationStartDate) -> None:
        self.__referenceLocationStartDate = (
            None if val is None else ReferenceLocationStartDate(val)
        )

    @property
    def referenceLocationEndDate(self) -> ReferenceLocationEndDate:
        return self.__referenceLocationEndDate

    @referenceLocationEndDate.setter
    def referenceLocationEndDate(self, val: ReferenceLocationEndDate) -> None:
        self.__referenceLocationEndDate = (
            None if val is None else ReferenceLocationEndDate(val)
        )

    @property
    def referenceLocationCitation(self) -> BibliographicReference:
        """
        Identifes the source that created or defined the Reference Location.
        """
        return self.__referenceLocationCitation

    @referenceLocationCitation.setter
    def referenceLocationCitation(self, val: BibliographicReference) -> None:
        """
        Identifes the source that created or defined the Reference Location.
        """
        self.__referenceLocationCitation = (
            None if val is None else BibliographicReference(val)
        )

    @property
    def commentText(self) -> CommentText:
        return self.__commentText

    @commentText.setter
    def commentText(self, val: CommentText) -> None:
        self.__commentText = None if val is None else CommentText(val)

    def generateXML(  # noqa: C901
        self, name: str = "ProjectMonitoringLocationWeighting"
    ) -> str:
        doc = Doc()
        asis = doc.asis
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__monitoringLocationIdentifier is None:
                raise WQXException(
                    "Attribute 'monitoringLocationIdentifier' is required."
                )
            line("MonitoringLocationIdentifier", self.__monitoringLocationIdentifier)
            if self.__locationWeightingFactorMeasure is None:
                raise WQXException("Attribute 'measureCompact' is required.")
            asis(
                self.__locationWeightingFactorMeasure.generateXML(
                    "LocationWeightingFactorMeasure"
                )
            )
            if self.__statisticalStratumText is not None:
                line("StatisticalStratumText", self.__statisticalStratumText)
            if self.__locationCategoryName is not None:
                line("LocationCategoryName", self.__locationCategoryName)
            if self.__locationStatusName is not None:
                line("LocationStatusName", self.__locationStatusName)
            if self.__referenceLocationTypeCode is not None:
                line("ReferenceLocationTypeCode", self.__referenceLocationTypeCode)
            if self.__referenceLocationStartDate is not None:
                line("ReferenceLocationStartDate", self.__referenceLocationStartDate)
            if self.__referenceLocationEndDate is not None:
                line("ReferenceLocationEndDate", self.__referenceLocationEndDate)
            if self.__referenceLocationCitation is not None:
                asis(
                    self.__referenceLocationCitation.generateXML(
                        "ReferenceLocationCitation"
                    )
                )
            if self.__commentText is not None:
                line("CommentText", self.__commentText)

        return doc.getvalue()



from yattag import Doc

from ..exceptions import WQXException
from .BibliographicReference import BibliographicReference
from .SimpleContent import (
    FormulaDescriptionText,
    MetricTypeIdentifier,
    MetricTypeIdentifierContext,
    MetricTypeName,
    MetricTypeScaleText,
)


class ActivityMetricType:
    """
    This section identifies the metric type reported as part of an activity metric.
    """

    __metricTypeIdentifier: MetricTypeIdentifier
    __metricTypeIdentifierContext: MetricTypeIdentifierContext
    __metricTypeName: MetricTypeName
    __metricTypeCitation: BibliographicReference
    __metricTypeScaleText: MetricTypeScaleText
    __formulaDescriptionText: FormulaDescriptionText

    def __init__(
        self,
        o: dict = None,
        *,
        metricTypeIdentifier: MetricTypeIdentifier = None,
        metricTypeIdentifierContext: MetricTypeIdentifierContext = None,
        metricTypeName: MetricTypeName = None,
        metricTypeCitation: BibliographicReference = None,
        metricTypeScaleText: MetricTypeScaleText = None,
        formulaDescriptionText: FormulaDescriptionText = None
    ):
        if isinstance(o, ActivityMetricType):
            # Assign attributes from object without typechecking
            self.__metricTypeIdentifier = o.metricTypeIdentifier
            self.__metricTypeIdentifierContext = o.metricTypeIdentifierContext
            self.__metricTypeName = o.metricTypeName
            self.__metricTypeCitation = o.metricTypeCitation
            self.__metricTypeScaleText = o.metricTypeScaleText
            self.__formulaDescriptionText = o.formulaDescriptionText
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.metricTypeIdentifier = o.get("metricTypeIdentifier")
            self.metricTypeIdentifierContext = o.get("metricTypeIdentifierContext")
            self.metricTypeName = o.get("metricTypeName")
            self.metricTypeCitation = o.get("metricTypeCitation")
            self.metricTypeScaleText = o.get("metricTypeScaleText")
            self.formulaDescriptionText = o.get("formulaDescriptionText")
        else:
            # Assign attributes from named keywords with typechecking
            self.metricTypeIdentifier = metricTypeIdentifier
            self.metricTypeIdentifierContext = metricTypeIdentifierContext
            self.metricTypeName = metricTypeName
            self.metricTypeCitation = metricTypeCitation
            self.metricTypeScaleText = metricTypeScaleText
            self.formulaDescriptionText = formulaDescriptionText

    @property
    def metricTypeIdentifier(self) -> MetricTypeIdentifier:
        return self.__metricTypeIdentifier

    @metricTypeIdentifier.setter
    def metricTypeIdentifier(self, val: MetricTypeIdentifier) -> None:
        self.__metricTypeIdentifier = MetricTypeIdentifier(val)

    @property
    def metricTypeIdentifierContext(self) -> MetricTypeIdentifierContext:
        return self.__metricTypeIdentifierContext

    @metricTypeIdentifierContext.setter
    def metricTypeIdentifierContext(self, val: MetricTypeIdentifierContext) -> None:
        self.__metricTypeIdentifierContext = MetricTypeIdentifierContext(val)

    @property
    def metricTypeName(self) -> MetricTypeName:
        return self.__metricTypeName

    @metricTypeName.setter
    def metricTypeName(self, val: MetricTypeName) -> None:
        self.__metricTypeName = None if val is None else MetricTypeName(val)

    @property
    def metricTypeCitation(self) -> BibliographicReference:
        """
        Provides additional description of the source that created or defined the
        metric.
        """
        return self.__metricTypeCitation

    @metricTypeCitation.setter
    def metricTypeCitation(self, val: BibliographicReference) -> None:
        """
        Provides additional description of the source that created or defined the
        metric.
        """
        self.__metricTypeCitation = None if val is None else BibliographicReference(val)

    @property
    def metricTypeScaleText(self) -> MetricTypeScaleText:
        return self.__metricTypeScaleText

    @metricTypeScaleText.setter
    def metricTypeScaleText(self, val: MetricTypeScaleText) -> None:
        self.__metricTypeScaleText = None if val is None else MetricTypeScaleText(val)

    @property
    def formulaDescriptionText(self) -> FormulaDescriptionText:
        return self.__formulaDescriptionText

    @formulaDescriptionText.setter
    def formulaDescriptionText(self, val: FormulaDescriptionText) -> None:
        self.__formulaDescriptionText = (
            None if val is None else FormulaDescriptionText(val)
        )

    def generateXML(self, name: str = "ActivityMetricType") -> str:
        doc = Doc()
        asis = doc.asis
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__metricTypeIdentifier is None:
                raise WQXException("Attribute 'metricTypeIdentifier' is required.")
            line("MetricTypeIdentifier", self.__metricTypeIdentifier)
            if self.__metricTypeIdentifierContext is None:
                raise WQXException("Attribute 'metricTypeIdentifierContext' is required.")
            line("MetricTypeIdentifierContext", self.__metricTypeIdentifierContext)
            if self.__metricTypeName is not None:
                line("MetricTypeName", self.__metricTypeName)
            if self.__metricTypeCitation is not None:
                asis(self.__metricTypeCitation.generateXML("MetricTypeCitation"))
            if self.__metricTypeScaleText is not None:
                line("MetricTypeScaleText", self.__metricTypeScaleText)
            if self.__formulaDescriptionText is not None:
                line("FormulaDescriptionText", self.__formulaDescriptionText)

        return doc.getvalue()



from typing import List, Union

from yattag import Doc

from ..exceptions import WQXException
from .BibliographicReference import BibliographicReference
from .SimpleContent import (
    CellFormName,
    CellShapeName,
    FunctionalFeedingGroupName,
    HabitName,
    TaxonomicPollutionTolerance,
    TaxonomicPollutionToleranceScaleText,
    TrophicLevelName,
    VoltinismName,
)


class TaxonomicDetails:
    """
    This section allows for the further definition of user-defined details for taxa.
    """

    __cellFormName: CellFormName
    __cellShapeName: CellShapeName
    __habitName: HabitName
    __voltinismName: VoltinismName
    __taxonomicPollutionTolerance: TaxonomicPollutionTolerance
    __taxonomicPollutionToleranceScaleText: TaxonomicPollutionToleranceScaleText
    __trophicLevelName: TrophicLevelName
    __functionalFeedingGroupName: FunctionalFeedingGroupName
    __taxonomicDetailsCitation: BibliographicReference

    def __init__(
        self,
        o: dict = None,
        *,
        cellFormName: CellFormName = None,
        cellShapeName: CellShapeName = None,
        habitName: HabitName = None,
        voltinismName: VoltinismName = None,
        taxonomicPollutionTolerance: TaxonomicPollutionTolerance = None,
        taxonomicPollutionToleranceScaleText: TaxonomicPollutionToleranceScaleText = None,
        trophicLevelName: TrophicLevelName = None,
        functionalFeedingGroupName: FunctionalFeedingGroupName = None,
        taxonomicDetailsCitation: BibliographicReference = None
    ):
        if isinstance(o, TaxonomicDetails):
            # Assign attributes from objects without typechecking
            self.__cellFormName = o.cellFormName
            self.__cellShapeName = o.cellShapeName
            self.__habitName = o.habitName
            self.__voltinismName = o.voltinismName
            self.__taxonomicPollutionTolerance = o.taxonomicPollutionTolerance
            self.__taxonomicPollutionToleranceScaleText = (
                o.taxonomicPollutionToleranceScaleText
            )
            self.__trophicLevelName = o.trophicLevelName
            self.__functionalFeedingGroupName = o.functionalFeedingGroupName
            self.__taxonomicDetailsCitation = o.taxonomicDetailsCitation
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.cellFormName = o.get("cellFormName")
            self.cellShapeName = o.get("cellShapeName")
            self.habitName = o.get("habitName")
            self.voltinismName = o.get("voltinismName")
            self.taxonomicPollutionTolerance = o.get("taxonomicPollutionTolerance")
            self.taxonomicPollutionToleranceScaleText = o.get(
                "taxonomicPollutionToleranceScaleText"
            )
            self.trophicLevelName = o.get("trophicLevelName")
            self.functionalFeedingGroupName = o.get("functionalFeedingGroupName")
            self.taxonomicDetailsCitation = o.get("taxonomicDetailsCitation")
        else:
            # Assign attributes from named keywords with typechecking
            self.cellFormName = cellFormName
            self.cellShapeName = cellShapeName
            self.habitName = habitName
            self.voltinismName = voltinismName
            self.taxonomicPollutionTolerance = taxonomicPollutionTolerance
            self.taxonomicPollutionToleranceScaleText = (
                taxonomicPollutionToleranceScaleText
            )
            self.trophicLevelName = trophicLevelName
            self.functionalFeedingGroupName = functionalFeedingGroupName
            self.taxonomicDetailsCitation = taxonomicDetailsCitation

    @property
    def cellFormName(self) -> CellFormName:
        return self.__cellFormName

    @cellFormName.setter
    def cellFormName(self, val: CellFormName) -> None:
        self.__cellFormName = None if val is None else CellFormName(val)

    @property
    def cellShapeName(self) -> CellShapeName:
        return self.__cellShapeName

    @cellShapeName.setter
    def cellShapeName(self, val: CellShapeName) -> None:
        self.__cellShapeName = None if val is None else CellShapeName(val)

    @property
    def habitName(self) -> HabitName:
        return self.__habitName

    @habitName.setter
    def habitName(self, val: Union[HabitName, List[HabitName]]) -> None:
        if val is None:
            self.__habitName = []
        elif isinstance(val, list):
            r: List[HabitName] = []
            for x in val:
                r.append(HabitName(x))
            self.__habitName = r
        else:
            self.__habitName = [HabitName(val)]

    @property
    def voltinismName(self) -> VoltinismName:
        return self.__voltinismName

    @voltinismName.setter
    def voltinismName(self, val: VoltinismName) -> None:
        self.__voltinismName = None if val is None else VoltinismName(val)

    @property
    def taxonomicPollutionTolerance(self) -> TaxonomicPollutionTolerance:
        return self.__taxonomicPollutionTolerance

    @taxonomicPollutionTolerance.setter
    def taxonomicPollutionTolerance(self, val: TaxonomicPollutionTolerance) -> None:
        self.__taxonomicPollutionTolerance = (
            None if val is None else TaxonomicPollutionTolerance(val)
        )

    @property
    def taxonomicPollutionToleranceScaleText(
        self,
    ) -> TaxonomicPollutionToleranceScaleText:
        return self.__taxonomicPollutionToleranceScaleText

    @taxonomicPollutionToleranceScaleText.setter
    def taxonomicPollutionToleranceScaleText(
        self, val: TaxonomicPollutionToleranceScaleText
    ) -> None:
        self.__taxonomicPollutionToleranceScaleText = (
            None if val is None else TaxonomicPollutionToleranceScaleText(val)
        )

    @property
    def trophicLevelName(self) -> TrophicLevelName:
        return self.__trophicLevelName

    @trophicLevelName.setter
    def trophicLevelName(self, val: TrophicLevelName) -> None:
        self.__trophicLevelName = None if val is None else TrophicLevelName(val)

    @property
    def functionalFeedingGroupName(self) -> FunctionalFeedingGroupName:
        return self.__functionalFeedingGroupName

    @functionalFeedingGroupName.setter
    def functionalFeedingGroupName(
        self, val: Union[FunctionalFeedingGroupName, List[FunctionalFeedingGroupName]]
    ) -> None:
        if val is None:
            self.__functionalFeedingGroupName = []
        elif isinstance(val, list):
            r: List[FunctionalFeedingGroupName] = []
            for x in val:
                r.append(FunctionalFeedingGroupName(x))
            self.__functionalFeedingGroupName = r
        else:
            self.__functionalFeedingGroupName = [FunctionalFeedingGroupName(val)]

    @property
    def taxonomicDetailsCitation(self) -> BibliographicReference:
        return self.__taxonomicDetailsCitation

    @taxonomicDetailsCitation.setter
    def taxonomicDetailsCitation(self, val: BibliographicReference) -> None:
        self.__taxonomicDetailsCitation = (
            None if val is None else BibliographicReference(val)
        )

    def generateXML(self, name: str = "TaxonomicDetails") -> str:  # noqa: C901
        doc = Doc()
        asis = doc.asis
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__cellFormName is not None:
                line("CellFormName", self.__cellFormName)
            if self.__cellShapeName is not None:
                line("CellShapeName", self.__cellShapeName)
            if len(self.__habitName) > 3:
                raise WQXException(
                    "Attribute 'habitName' must be a list of 0 to 3 HabitName objects."
                )
            for x in self.__habitName:
                line("HabitName", x)
            if self.__voltinismName is not None:
                line("VoltinismName", self.__voltinismName)
            if self.__taxonomicPollutionTolerance is not None:
                line("TaxonomicPollutionTolerance", self.__taxonomicPollutionTolerance)
            if self.__taxonomicPollutionToleranceScaleText is not None:
                line(
                    "TaxonomicPollutionToleranceScaleText",
                    self.__taxonomicPollutionToleranceScaleText,
                )
            if self.__trophicLevelName is not None:
                line("TrophicLevelName", self.__trophicLevelName)
            if len(self.__functionalFeedingGroupName) > 3:
                raise WQXException(
                    "Attribute 'functionalFeedingGroupName' must be a list of 0 to 3 "
                    "FunctionalFeedingGroupName objects."
                )
            for x in self.__functionalFeedingGroupName:
                line("FunctionalFeedingGroupName", x)
            if self.__taxonomicDetailsCitation is not None:
                asis(
                    self.__taxonomicDetailsCitation.generateXML(
                        "TaxonomicDetailsCitation"
                    )
                )

        return doc.getvalue()



from yattag import Doc

from .SimpleContent import ElectronicAddressText, ElectronicAddressTypeName


class ElectronicAddress:
    """
    A location within a system of worldwide electronic communication where a computer
    user can access information or receive electronic mail.
    """

    __electronicAddressText: ElectronicAddressText
    __electronicAddressTypeName: ElectronicAddressTypeName

    def __init__(
        self,
        o: dict = None,
        *,
        electronicAddressText: ElectronicAddressText = None,
        electronicAddressTypeName: ElectronicAddressTypeName = None
    ):
        if isinstance(o, ElectronicAddress):
            # Assign attributes from object without typechecking
            self.__electronicAddressText = o.electronicAddressText
            self.__electronicAddressTypeName = o.electronicAddressTypeName
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.electronicAddressText = o.get("electronicAddressText")
            self.electronicAddressTypeName = o.get("electronicAddressTypeName")
        else:
            # Assign attributes from named keywords with typechecking
            self.electronicAddressText = electronicAddressText
            self.electronicAddressTypeName = electronicAddressTypeName

    @property
    def electronicAddressText(self) -> ElectronicAddressText:
        return self.__electronicAddressText

    @electronicAddressText.setter
    def electronicAddressText(self, val: ElectronicAddressText) -> None:
        self.__electronicAddressText = None if val is None else ElectronicAddressText(val)

    @property
    def electronicAddressTypeName(self) -> ElectronicAddressTypeName:
        return self.__electronicAddressTypeName

    @electronicAddressTypeName.setter
    def electronicAddressTypeName(self, val: ElectronicAddressTypeName) -> None:
        self.__electronicAddressTypeName = (
            None if val is None else ElectronicAddressTypeName(val)
        )

    def generateXML(self, name: str = "ElectronicAddress") -> str:
        doc = Doc()
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__electronicAddressText is not None:
                line("ElectronicAddressText", self.__electronicAddressText)
            if self.__electronicAddressTypeName is not None:
                line("ElectronicAddressTypeName", self.__electronicAddressTypeName)

        return doc.getvalue()



import unittest
import xml.etree.cElementTree as ET

from wqxlib import Document, Header, Payload, Submission
from wqxlib.wqx_v3_0 import (
    WQX,
    MonitoringLocation,
    MonitoringLocationGeospatial,
    MonitoringLocationIdentity,
    Organization,
    OrganizationDescription,
)


class TestCreateWQXDocument(unittest.TestCase):
    def test_create_wqx_document(self):

        submission = Submission(
            document=Document(
                id="20201209ML8",
                header=Header(
                    author="Test Author",
                    organization="Test Organization",
                    contactInfo="Test Organization Mailing or Physical Address",
                    notification="test@example.org",
                ),
                # Even though we are assigning a single object to the payload key here,
                # it becomes a list item which can be appended to later
                payload=Payload(
                    operation="Update-Insert",
                    wqx=WQX(
                        organization=Organization(
                            organizationDescription=OrganizationDescription(
                                organizationIdentifier="WQXTEST",
                                organizationFormalName="WQX Test Organization",
                                organizationDescriptionText="Test",
                            ),
                            monitoringLocation=MonitoringLocation(
                                monitoringLocationGeospatial=MonitoringLocationGeospatial(
                                    latitudeMeasure="38.6470",
                                    longitudeMeasure="-82.8587",
                                    sourceMapScale="2400",
                                    horizontalCollectionMethodName="Interpolation-Map",
                                    horizontalCoordinateReferenceSystemDatumName="NAD83",
                                    countryCode="US",
                                    stateCode="WV",
                                    countyCode="039",
                                ),
                                monitoringLocationIdentity=MonitoringLocationIdentity(
                                    monitoringLocationIdentifier="GREENUP",
                                    monitoringLocationName="Greenup Dam",
                                    monitoringLocationTypeName="River/Stream",
                                    hucEightDigitCode="05090103",
                                    hucTwelveDigitCode="050901030107",
                                    tribalLandIndicator=False,
                                ),
                            ),
                        )
                    ),
                ),
            )
        )
        doc = submission.document.generateXML()

        tree = ET.ElementTree(ET.fromstring(doc))
        root = tree.getroot()
        self.assertEqual(
            root.tag,
            "{http://www.exchangenetwork.net/schema/v1.0/ExchangeNetworkDocument.xsd}Document",
        )
        header = root[0]
        self.assertEqual(
            header.tag,
            "{http://www.exchangenetwork.net/schema/v1.0/ExchangeNetworkDocument.xsd}Header",
        )
        self.assertEqual(header[0].text, "Test Author")
        self.assertEqual(header[1].text, "Test Organization")
        self.assertEqual(header[2].text, "WQX")
        self.assertEqual(header[4].text, "Test Organization Mailing or Physical Address")
        self.assertEqual(header[5].text, "test@example.org")
        payload = root[1]
        self.assertEqual(
            payload.tag,
            "{http://www.exchangenetwork.net/schema/v1.0/ExchangeNetworkDocument.xsd}Payload",
        )
        self.assertEqual(payload.attrib.get("Operation"), "Update-Insert")



from yattag import Doc

from ..exceptions import WQXException
from .SimpleContent import BinaryObjectFileName, BinaryObjectFileTypeCode


class AttachedBinaryObject:
    """
    Reference document, image, photo, GIS data layer, laboratory material or other
    electronic object attached within a data exchange, as well as information used to
    describe the object.
    """

    __binaryObjectFileName: BinaryObjectFileName
    __binaryObjectFileTypeCode: BinaryObjectFileTypeCode

    def __init__(
        self,
        o: dict = None,
        *,
        binaryObjectFileName: BinaryObjectFileName = None,
        binaryObjectFileTypeCode: BinaryObjectFileTypeCode = None
    ):
        if isinstance(o, AttachedBinaryObject):
            # Assign attributes from object without typechecking
            self.__binaryObjectFileName = o.binaryObjectFileName
            self.__binaryObjectFileTypeCode = o.binaryObjectFileTypeCode
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.binaryObjectFileName = o.get("binaryObjectFileName")
            self.binaryObjectFileTypeCode = o.get("binaryObjectFileTypeCode")
        else:
            # Assign attributes from named keywords with typechecking
            self.binaryObjectFileName = binaryObjectFileName
            self.binaryObjectFileTypeCode = binaryObjectFileTypeCode

    @property
    def binaryObjectFileName(self) -> BinaryObjectFileName:
        return self.__binaryObjectFileName

    @binaryObjectFileName.setter
    def binaryObjectFileName(self, val: BinaryObjectFileName) -> None:
        self.__binaryObjectFileName = BinaryObjectFileName(val)

    @property
    def binaryObjectFileTypeCode(self) -> BinaryObjectFileTypeCode:
        return self.__binaryObjectFileTypeCode

    @binaryObjectFileTypeCode.setter
    def binaryObjectFileTypeCode(self, val: BinaryObjectFileTypeCode) -> None:
        self.__binaryObjectFileTypeCode = BinaryObjectFileTypeCode(val)

    def generateXML(self, name: str = "AttachedBinaryObject") -> str:
        doc = Doc()
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__binaryObjectFileName is None:
                raise WQXException("Attribute 'binaryObjectFileName' is required.")
            line("BinaryObjectFileName", self.__binaryObjectFileName)
            if self.__binaryObjectFileTypeCode is None:
                raise WQXException("Attribute 'binaryObjectFileTypeCode' is required.")
            line("BinaryObjectFileTypeCode", self.__binaryObjectFileTypeCode)

        return doc.getvalue()



from .wqx_v3_0 import (
    ActivityDescription,
    ElectronicAddress,
    OrganizationAddress,
    OrganizationDescription,
    Telephonic,
)


class ImportConfiguration:
    __activityDescription: ActivityDescription
    __electronicAddress: ElectronicAddress
    __organizationAddress: OrganizationAddress
    __organizationDescription: OrganizationDescription
    __telephonic: Telephonic

    def __init__(
        self,
        o: dict = None,
        *,
        activityDescription: ActivityDescription = None,
        electronicAddress: ElectronicAddress = None,
        organizationAddress: OrganizationAddress = None,
        organizationDescription: OrganizationDescription = None,
        telephonic: Telephonic = None
    ):
        if isinstance(o, ImportConfiguration):
            # Assign attributes from object without typechecking
            self.__activityDescription = o.activityDescription
            self.__electronicAddress = o.electronicAddress
            self.__organizationAddress = o.organizationAddress
            self.__organizationDescription = o.organizationDescription
            self.__telephonic = o.telephonic
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.activityDescription = o.get("activityDescription")
            self.electronicAddress = o.get("electronicAddress")
            self.organizationAddress = o.get("organizationAddress")
            self.organizationDescription = o.get("organizationDescription")
            self.telephonic = o.get("telephonic")
        else:
            # Assign attributes from named keywords with typechecking
            self.activityDescription = activityDescription
            self.electronicAddress = electronicAddress
            self.organizationAddress = organizationAddress
            self.organizationDescription = organizationDescription
            self.telephonic = telephonic

    @property
    def activityDescription(self) -> ActivityDescription:
        return self.__activityDescription

    @activityDescription.setter
    def activityDescription(self, val: ActivityDescription) -> None:
        self.__activityDescription = None if val is None else ActivityDescription(val)

    @property
    def electronicAddress(self) -> ElectronicAddress:
        return self.__electronicAddress

    @electronicAddress.setter
    def electronicAddress(self, val: ElectronicAddress) -> None:
        self.__electronicAddress = None if val is None else ElectronicAddress(val)

    @property
    def organizationAddress(self) -> OrganizationAddress:
        return self.__organizationAddress

    @organizationAddress.setter
    def organizationAddress(self, val: OrganizationAddress) -> None:
        self.__organizationAddress = None if val is None else OrganizationAddress(val)

    @property
    def organizationDescription(self) -> OrganizationDescription:
        return self.__organizationDescription

    @organizationDescription.setter
    def organizationDescription(self, val: OrganizationDescription) -> None:
        self.__organizationDescription = (
            None if val is None else OrganizationDescription(val)
        )

    @property
    def telephonic(self) -> Telephonic:
        return self.__telephonic

    @telephonic.setter
    def telephonic(self, val: Telephonic) -> None:
        self.__telephonic = None if val is None else Telephonic(val)



from typing import List, Union

from yattag import Doc

from ..exceptions import WQXException
from .ActivityMetricType import ActivityMetricType
from .MeasureCompact import MeasureCompact
from .SimpleContent import (
    CommentText,
    IndexIdentifier,
    MetricSamplingPointPlaceInSeries,
    MetricScore,
)


class ActivityMetric:
    """
    This section allows for the reporting of metrics to support habitat or biotic
    integrity indices.
    """

    __activityMetricType: ActivityMetricType
    __metricValueMeasure: MeasureCompact
    __metricScore: MetricScore
    __metricSamplingPointPlaceInSeries: MetricSamplingPointPlaceInSeries
    __metricCommentText: CommentText
    __indexIdentifier: List[IndexIdentifier]

    def __init__(
        self,
        o: dict = None,
        *,
        activityMetricType: ActivityMetricType = None,
        metricValueMeasure: MeasureCompact = None,
        metricScore: MetricScore = None,
        metricSamplingPointPlaceInSeries: MetricSamplingPointPlaceInSeries = None,
        metricCommentText: CommentText = None,
        indexIdentifier: IndexIdentifier = None
    ):
        if isinstance(o, ActivityMetric):
            # Assign attributes from object without typechecking
            self.__activityMetricType = o.activityMetricType
            self.__metricValueMeasure = o.metricValueMeasure
            self.__metricScore = o.metricScore
            self.__metricSamplingPointPlaceInSeries = o.metricSamplingPointPlaceInSeries
            self.__metricCommentText = o.metricCommentText
            self.__indexIdentifier = o.indexIdentifier
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.activityMetricType = o.get("activityMetricType")
            self.metricValueMeasure = o.get("metricValueMeasure")
            self.metricScore = o.get("metricScore")
            self.metricSamplingPointPlaceInSeries = o.get(
                "metricSamplingPointPlaceInSeries"
            )
            self.metricCommentText = o.get("metricCommentText")
            self.indexIdentifier = o.get("indexIdentifier")
        else:
            # Assign attributes from named keywords with typechecking
            self.activityMetricType = activityMetricType
            self.metricValueMeasure = metricValueMeasure
            self.metricScore = metricScore
            self.metricSamplingPointPlaceInSeries = metricSamplingPointPlaceInSeries
            self.metricCommentText = metricCommentText
            self.indexIdentifier = indexIdentifier

    @property
    def activityMetricType(self) -> ActivityMetricType:
        return self.__activityMetricType

    @activityMetricType.setter
    def activityMetricType(self, val: ActivityMetricType) -> None:
        self.__activityMetricType = None if val is None else ActivityMetricType(val)

    @property
    def metricValueMeasure(self) -> MeasureCompact:
        """
        A non-scaled value calculated from raw results that may be scaled into a metric
        score.
        """
        return self.__metricValueMeasure

    @metricValueMeasure.setter
    def metricValueMeasure(self, val: MeasureCompact) -> None:
        """
        A non-scaled value calculated from raw results that may be scaled into a metric
        score.
        """
        self.__metricValueMeasure = None if val is None else MeasureCompact(val)

    @property
    def metricScore(self) -> MetricScore:
        return self.__metricScore

    @metricScore.setter
    def metricScore(self, val: MetricScore) -> None:
        self.__metricScore = None if val is None else MetricScore(val)

    @property
    def metricSamplingPointPlaceInSeries(self) -> MetricSamplingPointPlaceInSeries:
        return self.__metricSamplingPointPlaceInSeries

    @metricSamplingPointPlaceInSeries.setter
    def metricSamplingPointPlaceInSeries(self, val: MetricSamplingPointPlaceInSeries):
        self.__metricSamplingPointPlaceInSeries = (
            None if val is None else MetricSamplingPointPlaceInSeries(val)
        )

    @property
    def metricCommentText(self) -> CommentText:
        """
        Free text with general comments concerning the metric.
        """
        return self.__metricCommentText

    @metricCommentText.setter
    def metricCommentText(self, val: CommentText) -> None:
        """
        Free text with general comments concerning the metric.
        """
        self.__metricCommentText = None if val is None else CommentText(val)

    @property
    def indexIdentifier(self) -> IndexIdentifier:
        return self.__indexIdentifier

    @indexIdentifier.setter
    def indexIdentifier(self, val: Union[IndexIdentifier, List[IndexIdentifier]]) -> None:
        if val is None:
            self.__indexIdentifier = []
        elif isinstance(val, list):
            r: List[IndexIdentifier] = []
            for x in val:
                r.append(IndexIdentifier(x))
            self.__indexIdentifier = r
        else:
            self.__indexIdentifier = [IndexIdentifier(val)]

    def generateXML(self, name: str = "ActivityMetric") -> str:
        doc = Doc()
        asis = doc.asis
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__activityMetricType is None:
                raise WQXException("Attribute 'activityMetricType' is required.")
            asis(self.__activityMetricType.generateXML("ActivityMetricType"))
            if self.__metricValueMeasure is not None:
                asis(self.__metricValueMeasure.generateXML("MetricValueMeasure"))
            if self.__metricScore is None:
                raise WQXException("Attribute 'metricScore' is required.")
            line("MetricScore", self.__metricScore)
            if self.__metricSamplingPointPlaceInSeries is not None:
                line(
                    "MetricSamplingPointPlaceInSeries",
                    self.__metricSamplingPointPlaceInSeries,
                )
            if self.__metricCommentText is not None:
                line("MetricCommentText", self.__metricCommentText)
            for x in self.__indexIdentifier:
                line("IndexIdentifier", x)

        return doc.getvalue()



import sys


def main(argv: list):
    print(
        "This module does not yet support direct execution. It should be used as a library."
    )
    print(
        "More information is available at https://github.com/Flippingbinary/wqxlib/wqxlib-python"
    )
    return 0


if __name__ == "__main__":
    main(sys.argv[1:])



from enum import Enum

from yattag import Doc

from .exceptions import WQXException
from .wqx_v3_0.WQX import WQX
from .wqx_v3_0.WQX_Delete import WQXDelete
from .wqx_v3_0.WQX_Update_Identifiers import WQXUpdateIdentifiers


class OperationType(Enum):
    UPDATE_INSERT = "Update-Insert"
    DELETE = "Delete"


class Payload:
    """
    The Payload section of the document contains the WQX data.
    """

    __operation: OperationType
    __wqx: WQX
    __wqxUpdateIdentifiers: WQXUpdateIdentifiers
    __wqxDelete: WQXDelete

    UPDATE_INSERT = OperationType.UPDATE_INSERT
    DELETE = OperationType.DELETE

    def __init__(
        self,
        o: dict = None,
        *,
        operation=None,
        wqx=None,
        wqxUpdateIdentifiers=None,
        wqxDelete=None
    ):
        if isinstance(o, Payload):
            # Assign attributes from other Payload without typechecking
            self.__operation = o.operation
            self.__wqx = o.wqx
            self.__wqxUpdateIdentifiers = o.wqxUpdateIdentifiers
            self.__wqxDelete = o.wqxDelete
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.operation = o.get("operation")
            self.wqx = o.get("wqx")
            self.wqxUpdateIdentifiers = o.get("wqxUpdateIdentifiers")
            self.wqxDelete = o.get("wqxDelete")
        else:
            # Assign attributes from named keywords with typechecking
            self.operation = operation
            self.wqx = wqx
            self.wqxUpdateIdentifiers = wqxUpdateIdentifiers
            self.wqxDelete = wqxDelete

    @property
    def operation(self) -> OperationType:
        return self.__operation

    @operation.setter
    def operation(self, val: OperationType) -> None:
        self.__operation = None if val is None else OperationType(val)

    @property
    def wqx(self) -> WQX:
        return self.__wqx

    @wqx.setter
    def wqx(self, val: WQX) -> None:
        if val is not None and not isinstance(val, WQX):
            raise WQXException("Attribute 'wqx' must be a WQX object, if provided.")
        self.__wqx = None if val is None else WQX(val)

    @property
    def wqxUpdateIdentifiers(self) -> WQXUpdateIdentifiers:
        return self.__wqxUpdateIdentifiers

    @wqxUpdateIdentifiers.setter
    def wqxUpdateIdentifiers(self, val: WQXUpdateIdentifiers) -> None:
        if val is not None and not isinstance(val, WQXUpdateIdentifiers):
            raise WQXException(
                "Attribute 'wqxUpdateIdentifiers' must be a WQXUpdateIdentifiers object, "
                "if provided."
            )
        self.__wqxUpdateIdentifiers = None if val is None else WQXUpdateIdentifiers(val)

    @property
    def wqxDelete(self) -> WQXDelete:
        return self.__wqxDelete

    @wqxDelete.setter
    def wqxDelete(self, val: WQXDelete) -> None:
        if val is not None and not isinstance(val, WQXDelete):
            raise WQXException(
                "Attribute 'wqxDelete' must be a WQXDelete object, if provided."
            )
        self.__wqxDelete = None if val is None else WQXDelete(val)

    def generateXML(self, name: str = "Payload") -> str:  # noqa: C901
        doc = Doc()
        asis = doc.asis

        if self.__operation is None:
            raise WQXException("Attribute 'operation' is required.")
        with doc.tag(name, ("Operation", self.__operation.value)):
            if self.__operation == OperationType.UPDATE_INSERT:
                if self.__wqxDelete is not None:
                    raise WQXException(
                        "Attribute 'wqxDelete' must be set to None for 'Update-Insert' "
                        "operation."
                    )
                if self.__wqx is None and self.__wqxUpdateIdentifiers is None:
                    raise WQXException(
                        "One of attributes 'wqx' or 'wqxUpdateIdentifiers' are required "
                        "for 'Update-Insert' operation."
                    )
                elif self.__wqx is not None and self.__wqxUpdateIdentifiers is not None:
                    raise WQXException(
                        "One of attributes 'wqx' or 'wqxUpdateIdentifiers' must be set "
                        "to None for 'Update-Insert' operation."
                    )
                elif self.__wqx is None and self.__wqxUpdateIdentifiers is None:
                    raise WQXException(
                        "One of attributes 'wqx' or 'wqxUpdateIdentifiers' must be set "
                        "for 'Update-Insert' operation."
                    )
                elif self.__wqx is not None and self.__wqxUpdateIdentifiers is None:
                    asis(self.__wqx.generateXML("WQX"))
                elif self.__wqxUpdateIdentifiers is not None and self.__wqx is None:
                    asis(self.__wqxUpdateIdentifiers.generateXML("WQXUpdateIdentifiers"))
            elif self.__operation == OperationType.DELETE:
                if self.__wqx is not None:
                    raise WQXException(
                        "Attribute 'wqx' must be set to None for 'Delete' operation."
                    )
                if self.__wqxUpdateIdentifiers is not None:
                    raise WQXException(
                        "Attribute 'wqxUpdateIdentifiers' must be set to None for "
                        "'Delete' operation."
                    )
                if self.__wqxDelete is None:
                    raise WQXException(
                        "Attribute 'wqxDelete' is required for 'Delete' operation."
                    )
                asis(self.__wqxDelete.generateXML("WQXDelete"))
            else:
                raise WQXException(
                    "Attribute 'operation' must be either 'Update-Insert' or 'Delete'."
                )

        return doc.getvalue()



from yattag import Doc

from .BiologicalHabitatCollectionInformation import BiologicalHabitatCollectionInformation
from .SimpleContent import AssemblageSampledName, HabitatSelectionMethod, ToxicityTestType


class BiologicalActivityDescription:
    """
    Allows for the reporting of biological monitoring activities conducted at a
    Monitoring Location.
    """

    __assemblageSampledName: AssemblageSampledName
    __biologicalHabitatCollectionInformation: BiologicalHabitatCollectionInformation
    __toxicityTestType: ToxicityTestType
    __habitatSelectionMethod: HabitatSelectionMethod

    def __init__(
        self,
        o: dict = None,
        *,
        assemblageSampledName: AssemblageSampledName = None,
        biologicalHabitatCollectionInformation: BiologicalHabitatCollectionInformation = None,
        toxicityTestType: ToxicityTestType = None,
        habitatSelectionMethod: HabitatSelectionMethod = None
    ):
        if isinstance(o, BiologicalActivityDescription):
            # Assign attributes from object without typechecking
            self.__assemblageSampledName = o.assemblageSampledName
            self.__biologicalHabitatCollectionInformation = (
                o.biologicalHabitatCollectionInformation
            )
            self.__toxicityTestType = o.toxicityTestType
            self.__habitatSelectionMethod = o.habitatSelectionMethod
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.assemblageSampledName = o.get("assemblageSampledName")
            self.biologicalHabitatCollectionInformation = o.get(
                "biologicalHabitatCollectionInformation"
            )
            self.toxicityTestType = o.get("toxicityTestType")
            self.habitatSelectionMethod = o.get("habitatSelectionMethod")
        else:
            # Assign attributes from named keywords with typechecking
            self.assemblageSampledName = assemblageSampledName
            self.biologicalHabitatCollectionInformation = (
                biologicalHabitatCollectionInformation
            )
            self.toxicityTestType = toxicityTestType
            self.habitatSelectionMethod = habitatSelectionMethod

    @property
    def assemblageSampledName(self) -> AssemblageSampledName:
        return self.__assemblageSampledName

    @assemblageSampledName.setter
    def assemblageSampledName(self, val: AssemblageSampledName) -> None:
        self.__assemblageSampledName = None if val is None else AssemblageSampledName(val)

    @property
    def biologicalHabitatCollectionInformation(
        self,
    ) -> BiologicalHabitatCollectionInformation:
        return self.__biologicalHabitatCollectionInformation

    @biologicalHabitatCollectionInformation.setter
    def biologicalHabitatCollectionInformation(
        self, val: BiologicalHabitatCollectionInformation
    ) -> None:
        self.__biologicalHabitatCollectionInformation = (
            None if val is None else BiologicalHabitatCollectionInformation(val)
        )

    @property
    def toxicityTestType(self) -> ToxicityTestType:
        return self.__toxicityTestType

    @toxicityTestType.setter
    def toxicityTestType(self, val: ToxicityTestType) -> None:
        self.__toxicityTestType = None if val is None else ToxicityTestType(val)

    @property
    def habitatSelectionMethod(self) -> HabitatSelectionMethod:
        return self.__habitatSelectionMethod

    @habitatSelectionMethod.setter
    def habitatSelectionMethod(self, val: HabitatSelectionMethod) -> None:
        self.__habitatSelectionMethod = (
            None if val is None else HabitatSelectionMethod(val)
        )

    def generateXML(self, name: str = "BiologicalActivityDescription") -> str:
        doc = Doc()
        asis = doc.asis
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__assemblageSampledName is not None:
                line("AssemblageSampledName", self.__assemblageSampledName)
            if self.__biologicalHabitatCollectionInformation is not None:
                asis(
                    self.__biologicalHabitatCollectionInformation.generateXML(
                        "BiologicalHabitatCollectionInformation"
                    )
                )
            if self.__toxicityTestType is not None:
                line("ToxicityTestType", self.__toxicityTestType)
            if self.__habitatSelectionMethod is not None:
                line("HabitatSelectionMethod", self.__habitatSelectionMethod)

        return doc.getvalue()



from typing import List, Union

from yattag import Doc

from ..exceptions import WQXException
from .SimpleContent import NewIdentifier, OldIdentifier, OrganizationIdentifier


class IdentifierUpdate:
    """
    Allows a Project Identifier to be changed.
    """

    __oldIdentifier: OldIdentifier
    __newIdentifier: NewIdentifier

    def __init__(
        self,
        o: dict = None,
        *,
        oldIdentifier: OldIdentifier = None,
        newIdentifier: NewIdentifier = None
    ):
        if isinstance(o, IdentifierUpdate):
            # Assign attributes from object without typechecking
            self.__oldIdentifier = o.oldIdentifier
            self.__newIdentifier = o.newIdentifier
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.oldIdentifier = o.get("oldIdentifier")
            self.newIdentifier = o.get("newIdentifier")
        else:
            # Assign attributes from named keywords with typechecking
            self.oldIdentifier = oldIdentifier
            self.newIdentifier = newIdentifier

    @property
    def oldIdentifier(self) -> OldIdentifier:
        return self.__oldIdentifier

    @oldIdentifier.setter
    def oldIdentifier(self, val: OldIdentifier) -> None:
        self.__oldIdentifier = None if val is None else OldIdentifier(val)

    @property
    def newIdentifier(self) -> NewIdentifier:
        return self.__newIdentifier

    @newIdentifier.setter
    def newIdentifier(self, val: NewIdentifier) -> None:
        self.__newIdentifier = None if val is None else NewIdentifier(val)

    def generateXML(self, name="IdentifierUpdate"):
        doc = Doc()
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__oldIdentifier is None:
                raise WQXException("Attribute 'oldIdentifier' is required.")
            line("OldIdentifier", self.__oldIdentifier)
            if self.__newIdentifier is None:
                raise WQXException("Attribute 'newIdentifier' is required.")
            line("NewIdentifier", self.__newIdentifier)

        return doc.getvalue()


class UpdateIdentifiers:
    """
    Allows a set of identifiers to be changed.
    """

    __organizationIdentifier: OrganizationIdentifier
    __projectIdentifierUpdate: List[IdentifierUpdate]
    __monitoringLocationIdentifierUpdate: List[IdentifierUpdate]
    __indexIdentifierUpdate: List[IdentifierUpdate]
    __activityIdentifierUpdate: List[IdentifierUpdate]
    __activityGroupIdentifierUpdate: List[IdentifierUpdate]

    def __init__(
        self,
        o: dict = None,
        *,
        organizationIdentifier: OrganizationIdentifier = None,
        projectIdentifierUpdate: List[IdentifierUpdate] = None,
        monitoringLocationIdentifierUpdate: List[IdentifierUpdate] = None,
        indexIdentifierUpdate: List[IdentifierUpdate] = None,
        activityIdentifierUpdate: List[IdentifierUpdate] = None,
        activityGroupIdentifierUpdate: List[IdentifierUpdate] = None
    ):
        if isinstance(o, UpdateIdentifiers):
            # Assign attributes from object without typechecking
            self.__organizationIdentifier = o.organizationIdentifier
            self.__projectIdentifierUpdate = o.projectIdentifierUpdate
            self.__monitoringLocationIdentifierUpdate = (
                o.monitoringLocationIdentifierUpdate
            )
            self.__indexIdentifierUpdate = o.indexIdentifierUpdate
            self.__activityIdentifierUpdate = o.activityIdentifierUpdate
            self.__activityGroupIdentifierUpdate = o.activityGroupIdentifierUpdate
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.organizationIdentifier = o.get("organizationIdentifier")
            self.projectIdentifierUpdate = o.get("projectIdentifierUpdate", [])
            self.monitoringLocationIdentifierUpdate = o.get(
                "monitoringLocationIdentifierUpdate", []
            )
            self.indexIdentifierUpdate = o.get("indexIdentifierUpdate", [])
            self.activityIdentifierUpdate = o.get("activityIdentifierUpdate", [])
            self.activityGroupIdentifierUpdate = o.get(
                "activityGroupIdentifierUpdate", []
            )
        else:
            # Assign attributes from named keywords with typechecking
            self.organizationIdentifier = organizationIdentifier
            self.projectIdentifierUpdate = (
                projectIdentifierUpdate if projectIdentifierUpdate is not None else []
            )
            self.monitoringLocationIdentifierUpdate = (
                monitoringLocationIdentifierUpdate
                if monitoringLocationIdentifierUpdate is not None
                else []
            )
            self.indexIdentifierUpdate = (
                indexIdentifierUpdate if indexIdentifierUpdate is not None else []
            )
            self.activityIdentifierUpdate = (
                activityIdentifierUpdate if activityIdentifierUpdate is not None else []
            )
            self.activityGroupIdentifierUpdate = (
                activityGroupIdentifierUpdate
                if activityGroupIdentifierUpdate is not None
                else []
            )

    @property
    def organizationIdentifier(self) -> OrganizationIdentifier:
        return self.__organizationIdentifier

    @organizationIdentifier.setter
    def organizationIdentifier(self, val: OrganizationIdentifier) -> None:
        self.__organizationIdentifier = OrganizationIdentifier(val)

    @property
    def projectIdentifierUpdate(self) -> List[IdentifierUpdate]:
        return self.__projectIdentifierUpdate

    @projectIdentifierUpdate.setter
    def projectIdentifierUpdate(
        self, val: Union[IdentifierUpdate, List[IdentifierUpdate]]
    ) -> None:
        if val is None:
            self.__projectIdentifierUpdate = []
        elif isinstance(val, list):
            r: List[IdentifierUpdate] = []
            for x in val:
                r.append(IdentifierUpdate(x))
            self.__projectIdentifierUpdate = r
        else:
            self.__projectIdentifierUpdate = [IdentifierUpdate(val)]

    @property
    def monitoringLocationIdentifierUpdate(self) -> List[IdentifierUpdate]:
        return self.__monitoringLocationIdentifierUpdate

    @monitoringLocationIdentifierUpdate.setter
    def monitoringLocationIdentifierUpdate(
        self, val: Union[IdentifierUpdate, List[IdentifierUpdate]]
    ) -> None:
        if val is None:
            self.__monitoringLocationIdentifierUpdate = []
        elif isinstance(val, list):
            r: List[IdentifierUpdate] = []
            for x in val:
                r.append(IdentifierUpdate(x))
            self.__monitoringLocationIdentifierUpdate = r
        else:
            self.__monitoringLocationIdentifierUpdate = [IdentifierUpdate(val)]

    @property
    def indexIdentifierUpdate(self) -> List[IdentifierUpdate]:
        return self.__indexIdentifierUpdate

    @indexIdentifierUpdate.setter
    def indexIdentifierUpdate(
        self, val: Union[IdentifierUpdate, List[IdentifierUpdate]]
    ) -> None:
        if val is None:
            self.__indexIdentifierUpdate = []
        elif isinstance(val, list):
            r: List[IdentifierUpdate] = []
            for x in val:
                r.append(IdentifierUpdate(x))
            self.__indexIdentifierUpdate = r
        else:
            self.__indexIdentifierUpdate = [IdentifierUpdate(val)]

    @property
    def activityIdentifierUpdate(self) -> List[IdentifierUpdate]:
        return self.__activityIdentifierUpdate

    @activityIdentifierUpdate.setter
    def activityIdentifierUpdate(
        self, val: Union[IdentifierUpdate, List[IdentifierUpdate]]
    ) -> None:
        if val is None:
            self.__activityIdentifierUpdate = []
        elif isinstance(val, list):
            r: List[IdentifierUpdate] = []
            for x in val:
                r.append(IdentifierUpdate(x))
            self.__activityIdentifierUpdate = r
        else:
            self.__activityIdentifierUpdate = [IdentifierUpdate(val)]

    @property
    def activityGroupIdentifierUpdate(self) -> List[IdentifierUpdate]:
        return self.__activityGroupIdentifierUpdate

    @activityGroupIdentifierUpdate.setter
    def activityGroupIdentifierUpdate(
        self, val: Union[IdentifierUpdate, List[IdentifierUpdate]]
    ) -> None:
        if val is None:
            self.__activityGroupIdentifierUpdate = []
        elif isinstance(val, list):
            r: List[IdentifierUpdate] = []
            for x in val:
                r.append(IdentifierUpdate(x))
            self.__activityGroupIdentifierUpdate = r
        else:
            self.__activityGroupIdentifierUpdate = [IdentifierUpdate(val)]

    def generateXML(self, name="UpdateIdentifiers"):
        doc = Doc()
        asis = doc.asis
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__organizationIdentifier is None:
                raise WQXException("Attribute 'organizationIdentifier' is required.")
            line("OrganizationIdentifier", self.__organizationIdentifier)
            for x in self.__projectIdentifierUpdate:
                asis(x.generateXML("ProjectIdentifierUpdate"))
            for x in self.__monitoringLocationIdentifierUpdate:
                asis(x.generateXML("MonitoringLocationIdentifierUpdate"))
            for x in self.__indexIdentifierUpdate:
                asis(x.generateXML("IndexIdentifierUpdate"))
            for x in self.__activityIdentifierUpdate:
                asis(x.generateXML("ActivityIdentifierUpdate"))
            for x in self.__activityGroupIdentifierUpdate:
                asis(x.generateXML("ActivityGroupIdentifierUpdate"))

        return doc.getvalue()



from typing import List, Union

from yattag import Doc

from ..exceptions import WQXException
from .SimpleContent import MeasureQualifierCode, MeasureUnitCode, ResultMeasureValue


class Measure:
    """
    Identifies the value, associated units of measure, and qualifier for measuring the
    observation or analytical result value.
    """

    __resultMeasureValue: ResultMeasureValue
    __measureUnitCode: MeasureUnitCode
    __measureQualifierCode: List[MeasureQualifierCode]

    def __init__(
        self,
        o: dict = None,
        *,
        resultMeasureValue: ResultMeasureValue = None,
        measureUnitCode: MeasureUnitCode = None,
        measureQualifierCode: List[MeasureQualifierCode] = None
    ):
        if isinstance(o, Measure):
            # Assign attributes from object without typechecking
            self.__resultMeasureValue = o.resultMeasureValue
            self.__measureUnitCode = o.measureUnitCode
            self.__measureQualifierCode = o.measureQualifierCode
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.__resultMeasureValue = o.get("resultMeasureValue")
            self.__measureUnitCode = o.get("measureUnitCode")
            self.__measureQualifierCode = o.get("measureQualifierCode", [])
        else:
            # Assign attributes from named keywords with typechecking
            self.__resultMeasureValue = resultMeasureValue
            self.__measureUnitCode = measureUnitCode
            self.__measureQualifierCode = (
                measureQualifierCode if measureQualifierCode is not None else []
            )

    @property
    def resultMeasureValue(self) -> ResultMeasureValue:
        return self.__resultMeasureValue

    @resultMeasureValue.setter
    def resultMeasureValue(self, val: ResultMeasureValue) -> None:
        self.__resultMeasureValue = None if val is None else ResultMeasureValue(val)

    @property
    def measureUnitCode(self) -> MeasureUnitCode:
        return self.__measureUnitCode

    @measureUnitCode.setter
    def measureUnitCode(self, val: MeasureUnitCode) -> None:
        self.__measureUnitCode = None if val is None else MeasureUnitCode(val)

    @property
    def measureQualifierCode(self) -> List[MeasureQualifierCode]:
        return self.__measureQualifierCode

    @measureQualifierCode.setter
    def measureQualifierCode(
        self, val: Union[MeasureQualifierCode, List[MeasureQualifierCode]]
    ) -> None:
        if val is None:
            self.__measureQualifierCode = []
        elif isinstance(val, list):
            r: List[MeasureQualifierCode] = []
            for x in val:
                r.append(MeasureQualifierCode(x))
            self.__measureQualifierCode = r
        else:
            self.__measureQualifierCode = [MeasureQualifierCode(val)]

    def generateXML(self, name: str = "Measure") -> str:
        doc = Doc()
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__resultMeasureValue is not None:
                line("ResultMeasureValue", self.__resultMeasureValue)
            if self.__measureUnitCode is not None:
                line("MeasureUnitCode", self.__measureUnitCode)
            if len(self.__measureQualifierCode) > 6:
                raise WQXException(
                    "Attribute 'measureQualifierCode' must be a list of 0 to 6 "
                    "MeasureQualifierCode objects."
                )
            for x in self.__measureQualifierCode:
                line("MeasureQualifierCode", x)

        return doc.getvalue()



from .wqx_v3_0 import Telephonic


class WQXTelephonic(Telephonic):
    def __init__(self,):
        Telephonic.__init__(self)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass  # TODO: add a rule test



from yattag import Doc

from .SimpleContent import (
    TelephoneExtensionNumberText,
    TelephoneNumberText,
    TelephoneNumberTypeName,
)


class Telephonic:
    """
    An identification of a telephone connection.
    """

    __telephoneNumberText: TelephoneNumberText
    __telephoneNumberTypeName: TelephoneNumberTypeName
    __telephoneExtensionNumberText: TelephoneExtensionNumberText

    def __init__(
        self,
        o: dict = None,
        *,
        telephoneNumberText: TelephoneNumberText = None,
        telephoneNumberTypeName: TelephoneNumberTypeName = None,
        telephoneExtensionNumberText: TelephoneExtensionNumberText = None
    ):
        if isinstance(o, Telephonic):
            # Assign attributes from object without typechecking
            self.__telephoneNumberText = o.telephoneNumberText
            self.__telephoneNumberTypeName = o.telephoneNumberTypeName
            self.__telephoneExtensionNumberText = o.telephoneExtensionNumberText
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.telephoneNumberText = o.get("telephoneNumberText")
            self.telephoneNumberTypeName = o.get("telephoneNumberTypeName")
            self.telephoneExtensionNumberText = o.get("telephoneExtensionNumberText")
        else:
            # Assign attributes from named keywords with typechecking
            self.telephoneNumberText = telephoneNumberText
            self.telephoneNumberTypeName = telephoneNumberTypeName
            self.telephoneExtensionNumberText = telephoneExtensionNumberText

    @property
    def telephoneNumberText(self) -> TelephoneNumberText:
        return self.__telephoneNumberText

    @telephoneNumberText.setter
    def telephoneNumberText(self, val: TelephoneNumberText) -> None:
        self.__telephoneNumberText = None if val is None else TelephoneNumberText(val)

    @property
    def telephoneNumberTypeName(self) -> TelephoneNumberTypeName:
        return self.__telephoneNumberTypeName

    @telephoneNumberTypeName.setter
    def telephoneNumberTypeName(self, val: TelephoneNumberTypeName) -> None:
        self.__telephoneNumberTypeName = (
            None if val is None else TelephoneNumberTypeName(val)
        )

    @property
    def telephoneExtensionNumberText(self) -> TelephoneExtensionNumberText:
        return self.__telephoneExtensionNumberText

    @telephoneExtensionNumberText.setter
    def telephoneExtensionNumberText(self, val: TelephoneExtensionNumberText) -> None:
        self.__telephoneExtensionNumberText = (
            None if val is None else TelephoneExtensionNumberText(val)
        )

    def generateXML(self, name: str = "Telephonic") -> str:
        doc = Doc()
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__telephoneNumberText is not None:
                line("TelephoneNumberText", self.__telephoneNumberText)
            if self.__telephoneNumberTypeName is not None:
                line("TelephoneNumberTypeName", self.__telephoneNumberTypeName)
            if self.__telephoneExtensionNumberText is not None:
                line("TelephoneExtensionNumberText", self.__telephoneExtensionNumberText)

        return doc.getvalue()



from yattag import Doc

from ..exceptions import WQXException
from .ReferenceMethod import ReferenceMethod
from .SamplePreparation import SamplePreparation
from .SimpleContent import (
    HydrologicCondition,
    HydrologicEvent,
    SampleCollectionEquipmentCommentText,
    SampleCollectionEquipmentName,
)


class SampleDescription:
    """
    Basic identification information for the sample collected as part of a monitoring
    activity.
    """

    __sampleCollectionMethod: ReferenceMethod = None
    __sampleCollectionEquipmentName: SampleCollectionEquipmentName = None
    __sampleCollectionEquipmentCommentText: SampleCollectionEquipmentCommentText = None
    __samplePreparation: SamplePreparation = None
    __hydrologicCondition: HydrologicCondition = None
    __hydrologicEvent: HydrologicEvent = None

    def __init__(
        self,
        o: dict = None,
        *,
        sampleCollectionMethod: ReferenceMethod = None,
        sampleCollectionEquipmentName: SampleCollectionEquipmentName = None,
        sampleCollectionEquipmentCommentText: SampleCollectionEquipmentCommentText = None,
        samplePreparation: SamplePreparation = None,
        hydrologicCondition: HydrologicCondition = None,
        hydrologicEvent: HydrologicEvent = None
    ):
        if isinstance(o, SampleDescription):
            # Assign attributes from object without typechecking
            self.__sampleCollectionMethod = o.sampleCollectionMethod
            self.__sampleCollectionEquipmentName = o.sampleCollectionEquipmentName
            self.__sampleCollectionEquipmentCommentText = (
                o.sampleCollectionEquipmentCommentText
            )
            self.__samplePreparation = o.samplePreparation
            self.__hydrologicCondition = o.hydrologicCondition
            self.__hydrologicEvent = o.hydrologicEvent
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.sampleCollectionMethod = o.get("sampleCollectionMethod")
            self.sampleCollectionEquipmentName = o.get("sampleCollectionEquipmentName")
            self.sampleCollectionEquipmentCommentText = o.get(
                "sampleCollectionEquipmentCommentText"
            )
            self.samplePreparation = o.get("samplePreparation")
            self.hydrologicCondition = o.get("hydrologicCondition")
            self.hydrologicEvent = o.get("hydrologicEvent")
        else:
            # Assign attributes from named keywords with typechecking
            self.sampleCollectionMethod = sampleCollectionMethod
            self.sampleCollectionEquipmentName = sampleCollectionEquipmentName
            self.sampleCollectionEquipmentCommentText = (
                sampleCollectionEquipmentCommentText
            )
            self.samplePreparation = samplePreparation
            self.hydrologicCondition = hydrologicCondition
            self.hydrologicEvent = hydrologicEvent

    @property
    def sampleCollectionMethod(self) -> ReferenceMethod:
        """
        Identifies sample collection or measurement method procedures. Where a
        documented sample collection method has been employed, this enables the data
        provider to indicate the documented method that was employed during the field
        sample collection. Otherwise, the sample collection procedure will best be
        described in a freeform text.
        """
        return self.__sampleCollectionMethod

    @sampleCollectionMethod.setter
    def sampleCollectionMethod(self, val: ReferenceMethod) -> None:
        """
        Identifies sample collection or measurement method procedures. Where a documented
        sample collection method has been employed, this enables the data provider to
        indicate the documented method that was employed during the field sample
        collection. Otherwise, the sample collection procedure will best be described in
        a freeform text.
        """
        self.__sampleCollectionMethod = None if val is None else ReferenceMethod(val)

    @property
    def sampleCollectionEquipmentName(self) -> SampleCollectionEquipmentName:
        return self.__sampleCollectionEquipmentName

    @sampleCollectionEquipmentName.setter
    def sampleCollectionEquipmentName(self, val: SampleCollectionEquipmentName) -> None:
        self.__sampleCollectionEquipmentName = (
            None if val is None else SampleCollectionEquipmentName(val)
        )

    @property
    def sampleCollectionEquipmentCommentText(
        self,
    ) -> SampleCollectionEquipmentCommentText:
        return self.__sampleCollectionEquipmentCommentText

    @sampleCollectionEquipmentCommentText.setter
    def sampleCollectionEquipmentCommentText(
        self, val: SampleCollectionEquipmentCommentText
    ) -> None:
        self.__sampleCollectionEquipmentCommentText = (
            None if val is None else SampleCollectionEquipmentCommentText(val)
        )

    @property
    def samplePreparation(self) -> SamplePreparation:
        return self.__samplePreparation

    @samplePreparation.setter
    def samplePreparation(self, val: SamplePreparation) -> None:
        self.__samplePreparation = None if val is None else SamplePreparation(val)

    @property
    def hydrologicCondition(self) -> HydrologicCondition:
        return self.__hydrologicCondition

    @hydrologicCondition.setter
    def hydrologicCondition(self, val: HydrologicCondition) -> None:
        self.__hydrologicCondition = None if val is None else HydrologicCondition(val)

    @property
    def hydrologicEvent(self) -> HydrologicEvent:
        return self.__hydrologicEvent

    @hydrologicEvent.setter
    def hydrologicEvent(self, val: HydrologicEvent) -> None:
        self.__hydrologicEvent = None if val is None else HydrologicEvent(val)

    def generateXML(self, name: str = "SampleDescription") -> str:
        doc = Doc()
        asis = doc.asis
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__sampleCollectionMethod is not None:
                asis(self.__sampleCollectionMethod.generateXML("SampleCollectionMethod"))
            if self.__sampleCollectionEquipmentName is None:
                raise WQXException(
                    "Attribute 'sampleCollectionEquipmentName' is required."
                )
            line("SampleCollectionEquipmentName", self.__sampleCollectionEquipmentName)
            if self.__sampleCollectionEquipmentCommentText is not None:
                line(
                    "SampleCollectionEquipmentCommentText",
                    self.__sampleCollectionEquipmentCommentText,
                )
            if self.__samplePreparation is not None:
                asis(self.__samplePreparation.generateXML("SamplePreparation"))
            if self.__hydrologicCondition is not None:
                line("HydrologicCondition", self.__hydrologicCondition)
            if self.__hydrologicEvent is not None:
                line("HydrologicEvent", self.__hydrologicEvent)

        return doc.getvalue()



from yattag import Doc

from ..exceptions import WQXException
from .SimpleContent import MeasureUnitCode, MeasureValue


class MeasureCompact:
    """
    Identifies only the value and the associated units of measure for measuring the
    observation or analytical result value.
    """

    __measureValue: MeasureValue
    __measureUnitCode: MeasureUnitCode

    def __init__(
        self,
        o: dict = None,
        *,
        measureValue: MeasureValue = None,
        measureUnitCode: MeasureUnitCode = None
    ):
        if isinstance(o, MeasureCompact):
            # Assign attributes from object without typechecking
            self.__measureValue = o.measureValue
            self.__measureUnitCode = o.measureUnitCode
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.measureValue = o.get("measureValue")
            self.measureUnitCode = o.get("measureUnitCode")
        else:
            # Assign attributes from named keywords with typechecking
            self.measureValue = measureValue
            self.measureUnitCode = measureUnitCode

    @property
    def measureValue(self) -> MeasureValue:
        return self.__measureValue

    @measureValue.setter
    def measureValue(self, val: MeasureValue) -> None:
        self.__measureValue = None if val is None else MeasureValue(val)

    @property
    def measureUnitCode(self) -> MeasureUnitCode:
        return self.__measureUnitCode

    @measureUnitCode.setter
    def measureUnitCode(self, val: MeasureUnitCode) -> None:
        self.__measureUnitCode = None if val is None else MeasureUnitCode(val)

    def generateXML(self, name: str = "MeasureCompact") -> str:
        doc = Doc()
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__measureValue is None:
                raise WQXException("Attribute 'measureValue' is required.")
            line("MeasureValue", self.__measureValue)
            if self.__measureUnitCode is None:
                raise WQXException("Attribute 'measureUnitCode' is required.")
            line("MeasureUnitCode", self.__measureUnitCode)

        return doc.getvalue()



from datetime import datetime
from typing import List, Union

from yattag import Doc

from .exceptions import WQXException


class Header:
    __author: str = None
    __organization: str = None
    __title: str = None
    __creationTime: datetime = None
    __comment: str = None
    __dataService: str = None
    __contactInfo: str = None
    __notification: List[str] = []
    __sensitivity: str = None
    __property: dict = {}

    def __init__(
        self,
        o: dict = None,
        *,
        author=None,
        organization=None,
        title=None,
        creationTime=None,
        comment=None,
        dataService=None,
        contactInfo=None,
        notification=None,
        sensitivity=None,
        property=None
    ):
        if isinstance(o, Header):
            # Assign attributes from object without typechecking
            self.__author = o.author
            self.__organization = o.organization
            self.__title = o.title
            self.__creationTime = o.creationTime
            self.__comment = o.comment
            self.__dataService = o.dataService
            self.__contactInfo = o.contactInfo
            self.__notification = o.notification
            self.__sensitivity = o.sensitivity
            self.__property = o.property
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.author = o.get("author")
            self.organization = o.get("organization")
            self.title = o.get("title")
            self.creationTime = o.get("creationTime")
            self.comment = o.get("comment")
            self.dataService = o.get("dataService")
            self.contactInfo = o.get("contactInfo")
            notification = o.get("notification")
            if notification is not None and not isinstance(notification, list):
                raise TypeError(
                    "Property 'notification' must be a list of 0 or more strings."
                )
            self.sensitivity = o.get("sensitivity")
            # self.property is tricky because Python confuses it with the decorator
            property = o.get("property")
            if property is not None and not isinstance(property, dict):
                raise TypeError(
                    "Property 'property' must be a dict of 0 or more key/value pairs."
                )
            self.__property = {} if property is None else property
        else:
            # Assign attributes from named keywords with typechecking
            self.author = author
            self.organization = organization
            self.title = title
            self.creationTime = creationTime
            self.comment = comment
            self.dataService = dataService
            self.contactInfo = contactInfo
            self.notification = notification
            self.sensitivity = sensitivity
            # self.property is tricky because Python confuses it with the decorator
            if property is not None and not isinstance(property, dict):
                raise TypeError(
                    "Property 'property' must be a dict of 0 or more key/value pairs."
                )
            self.__property = {} if property is None else property

    @property
    def author(self) -> str:
        return self.__author

    @author.setter
    def author(self, val: str) -> None:
        self.__author = None if val is None else str(val)

    @property
    def organization(self) -> str:
        return self.__organization

    @organization.setter
    def organization(self, val: str) -> None:
        if val is None:
            self.__organization = None
        else:
            if not isinstance(val, str):
                raise TypeError("Property 'organization' must be a string.")
            if len(val) < 1:
                raise ValueError("Property 'organization is required.")
            self.__organization = str(val)

    @property
    def title(self) -> str:
        return self.__title

    @title.setter
    def title(self, val: str) -> None:
        if val is not None and not isinstance(val, str):
            raise TypeError("Property 'title' must be a string")
        self.__title = "WQX" if val is None or len(val) < 1 else val

    @property
    def creationTime(self) -> datetime:
        return self.__creationTime

    @creationTime.setter
    def creationTime(self, val: datetime) -> None:
        if val is not None and not isinstance(val, datetime):
            raise TypeError("Property 'creationTime' must be a datetime.")
        self.__creationTime = datetime.now() if val is None else val

    @property
    def comment(self) -> str:
        return self.__comment

    @comment.setter
    def comment(self, val) -> None:
        if val is not None and not isinstance(val, str):
            raise TypeError("Property 'comment' must be a string, if provided.")
        self.__comment = None if val is None or len(val) < 1 else val

    @property
    def dataService(self) -> str:
        return self.__dataService

    @dataService.setter
    def dataService(self, val) -> None:
        if val is not None and not isinstance(val, str):
            raise TypeError("Property 'dataService' must be a string, if provided.")
        self.__dataService = None if val is None or len(val) < 1 else val

    @property
    def contactInfo(self) -> str:
        return self.__contactInfo

    @contactInfo.setter
    def contactInfo(self, val: str) -> None:
        self.__contactInfo = None if val is None else str(val)

    @property
    def notification(self) -> List[str]:
        return self.__notification

    @notification.setter
    def notification(self, val: Union[str, List[str]]) -> None:
        if val is None:
            self.__notification = []
        elif isinstance(val, list):
            r: List[str] = []
            for x in val:
                if not isinstance(x, str):
                    raise WQXException(
                        "Attribute 'notification' must be a list of valid URIs or a "
                        "single URI, if provided."
                    )
                r.append(x)
            self.__notification = r
        else:
            if not isinstance(val, str):
                raise WQXException(
                    "Attribute 'notification' must be a list of valid URIs or a single "
                    "URI, if provided."
                )
            self.__notification = [val]

    @property
    def sensitivity(self) -> str:
        return self.__sensitivity

    @sensitivity.setter
    def sensitivity(self, val) -> None:
        if val is not None and not isinstance(val, str):
            raise TypeError("Property 'sensitivity' must be a string, if provided.")
        self.__sensitivity = None if val is None or len(val) < 1 else val

    @property
    def property(self) -> dict:
        return self.__property

    @property.setter
    def property(self, val: dict) -> None:
        if val is not None and not isinstance(val, dict):
            raise TypeError(
                "Property 'property' must be a dict of 0 or more key/value pairs."
            )
        self.__property = {} if val is None else val

    def generateXML(self, name: str = "Header") -> str:  # noqa: C901
        doc = Doc()
        tag = doc.tag
        line = doc.line

        with tag(name):
            if self.__author is None:
                raise WQXException("Property 'author' is required.")
            line("Author", self.__author)
            if self.__organization is None:
                raise WQXException("Property 'organization' is required.")
            line("Organization", self.__organization)
            if self.__title is None:
                raise WQXException("Property 'title' is required.")
            line("Title", self.__title)
            if self.__creationTime is None:
                raise WQXException("Attribute 'creationTime' is required.")
            line(
                "CreationTime",
                self.__creationTime.astimezone().replace(microsecond=0).isoformat(),
            )
            if self.__comment is not None:
                line("Comment", self.__comment)
            if self.__dataService is not None:
                line("DataService", self.__dataService)
            if self.__contactInfo is not None:
                line("ContactInfo", self.__contactInfo)
            for x in self.__notification:
                line("Notification", x)
            if self.__sensitivity is not None:
                line("Sensitivity", self.__sensitivity)
            for key in self.__property:
                with tag("Property"):
                    line("name", key)
                    line("value", self.__property[key])

        return doc.getvalue()



import datetime

from yattag import Doc

from ..exceptions import WQXException
from .SimpleContent import Time, TimeZoneCode


class WQXTime:
    """
    Custom WQX datatype that defines a local time and corresponding time zone in which
    the time is measured.
    """

    __time: Time = None
    __timeZoneCode: TimeZoneCode = None

    def __init__(
        self, o: dict = None, *, time: Time = None, timeZoneCode: TimeZoneCode = None
    ):
        if isinstance(o, WQXTime):
            # Assign attributes from object without typechecking
            self.__time = o.time
            self.__timeZoneCode = o.timeZoneCode
        elif isinstance(o, datetime.datetime):
            self.__time = o.time()
            self.__timeZoneCode = o.strftime("%Z")
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.time = o.get("time")
            self.timeZoneCode = o.get("timeZoneCode")
        else:
            # Assign attributes from named keywords with typechecking
            self.time = time
            self.timeZoneCode = timeZoneCode

    @property
    def time(self) -> Time:
        return self.__time

    @time.setter
    def time(self, val: Time) -> None:
        self.__time = None if val is None else Time(val)

    @property
    def timeZoneCode(self) -> TimeZoneCode:
        return self.__timeZoneCode

    @timeZoneCode.setter
    def timeZoneCode(self, val: TimeZoneCode) -> None:
        self.__timeZoneCode = None if val is None else TimeZoneCode(val)

    def generateXML(self, name: str = "WQXTime") -> str:
        doc = Doc()
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__time is None:
                raise WQXException("Attribute 'time' is required.")
            line("Time", self.__time.strftime("%H:%M:%S"))
            if self.__timeZoneCode is None:
                raise WQXException("Attribute 'timeZoneCode' is required.")
            line("TimeZoneCode", self.__timeZoneCode)

        return doc.getvalue()



from yattag import Doc

from ..exceptions import WQXException
from .IndexType import IndexType
from .SimpleContent import (
    CommentText,
    IndexCalculatedDate,
    IndexIdentifier,
    IndexQualifierCode,
    IndexScore,
    MonitoringLocationIdentifier,
)


class BiologicalHabitatIndex:
    """
    This section allows for the reporting of habitat and biotic integrity indices as a
    representation of water quality conditions.
    """

    __indexIdentifier: IndexIdentifier
    __indexType: IndexType
    __indexScore: IndexScore
    __indexQualifierCode: IndexQualifierCode
    __indexCommentText: CommentText
    __indexCalculatedDate: IndexCalculatedDate
    __monitoringLocationIdentifier: MonitoringLocationIdentifier

    def __init__(
        self,
        o: dict = None,
        *,
        indexIdentifier: IndexIdentifier = None,
        indexType: IndexType = None,
        indexScore: IndexScore = None,
        indexQualifierCode: IndexQualifierCode = None,
        indexCommentText: CommentText = None,
        indexCalculatedDate: IndexCalculatedDate = None,
        monitoringLocationIdentifier: MonitoringLocationIdentifier = None
    ):
        if isinstance(o, BiologicalHabitatIndex):
            # Assign attributes from object without typechecking
            self.__indexIdentifier = o.indexIdentifier
            self.__indexType = o.indexType
            self.__indexScore = o.indexScore
            self.__indexQualifierCode = o.indexQualifierCode
            self.__indexCommentText = o.indexCommentText
            self.__indexCalculatedDate = o.indexCalculatedDate
            self.__monitoringLocationIdentifier = o.monitoringLocationIdentifier
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.indexIdentifier = o.get("indexIdentifier")
            self.indexType = o.get("indexType")
            self.indexScore = o.get("indexScore")
            self.indexQualifierCode = o.get("indexQualifierCode")
            self.indexCommentText = o.get("indexCommentText")
            self.indexCalculatedDate = o.get("indexCalculatedDate")
            self.monitoringLocationIdentifier = o.get("monitoringLocationIdentifier")
        else:
            # Assign attributes from named keywords with typechecking
            self.indexIdentifier = indexIdentifier
            self.indexType = indexType
            self.indexScore = indexScore
            self.indexQualifierCode = indexQualifierCode
            self.indexCommentText = indexCommentText
            self.indexCalculatedDate = indexCalculatedDate
            self.monitoringLocationIdentifier = monitoringLocationIdentifier

    @property
    def indexIdentifier(self) -> IndexIdentifier:
        return self.__indexIdentifier

    @indexIdentifier.setter
    def indexIdentifier(self, val: IndexIdentifier) -> None:
        self.__indexIdentifier = IndexIdentifier(val)

    @property
    def indexType(self) -> IndexType:
        return self.__indexType

    @indexType.setter
    def indexType(self, val: IndexType) -> None:
        self.__indexType = None if val is None else IndexType(val)

    @property
    def indexScore(self) -> IndexScore:
        return self.__indexScore

    @indexScore.setter
    def indexScore(self, val: IndexScore) -> None:
        self.__indexScore = IndexScore(val)

    @property
    def indexQualifierCode(self) -> IndexQualifierCode:
        return self.__indexQualifierCode

    @indexQualifierCode.setter
    def indexQualifierCode(self, val: IndexQualifierCode) -> None:
        self.__indexQualifierCode = None if val is None else IndexQualifierCode(val)

    @property
    def indexCommentText(self) -> CommentText:
        """
        Free text with general comments concerning the index.
        """
        return self.__indexCommentText

    @indexCommentText.setter
    def indexCommentText(self, val: CommentText) -> None:
        """
        Free text with general comments concerning the index.
        """
        self.__indexCommentText = None if val is None else CommentText(val)

    @property
    def indexCalculatedDate(self) -> IndexCalculatedDate:
        return self.__indexCalculatedDate

    @indexCalculatedDate.setter
    def indexCalculatedDate(self, val: IndexCalculatedDate) -> None:
        self.__indexCalculatedDate = None if val is None else IndexCalculatedDate(val)

    @property
    def monitoringLocationIdentifier(self) -> MonitoringLocationIdentifier:
        return self.__monitoringLocationIdentifier

    @monitoringLocationIdentifier.setter
    def monitoringLocationIdentifier(self, val: MonitoringLocationIdentifier) -> None:
        self.__monitoringLocationIdentifier = MonitoringLocationIdentifier(val)

    def generateXML(self, name: str = "BiologicalHabitatIndex") -> str:
        doc = Doc()
        asis = doc.asis
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__indexIdentifier is None:
                raise WQXException("Attribute 'indexIdentifier' is required.")
            line("IndexIdentifier", self.__indexIdentifier)
            if self.__indexType is None:
                raise WQXException("Attribute 'indexType' is required.")
            asis(self.__indexType.generateXML("IndexType"))
            if self.__indexScore is None:
                raise WQXException("Attribute 'indexScore' is required.")
            line("IndexScore", self.__indexScore)
            if self.__indexQualifierCode is not None:
                line("IndexQualifierCode", self.__indexQualifierCode)
            if self.__indexCommentText is not None:
                line("IndexCommentText", self.__indexCommentText)
            if self.__indexCalculatedDate is not None:
                line("IndexCalculatedDate", self.__indexCalculatedDate)
            if self.__monitoringLocationIdentifier is None:
                raise WQXException(
                    "Attribute 'monitoringLocationIdentifier' is required."
                )
            line("MonitoringLocationIdentifier", self.__monitoringLocationIdentifier)

        return doc.getvalue()



from yattag import Doc

from ..exceptions import WQXException
from .MeasureCompact import MeasureCompact
from .SimpleContent import (
    CountryCode,
    CountyCode,
    HorizontalCollectionMethodName,
    HorizontalCoordinateReferenceSystemDatumName,
    LatitudeMeasure,
    LongitudeMeasure,
    SourceMapScale,
    StateCode,
    VerticalCollectionMethodName,
    VerticalCoordinateReferenceSystemDatumName,
)


class MonitoringLocationGeospatial:
    """
    Monitoring location geographic location.
    """

    __latitudeMeasure: LatitudeMeasure
    __longitudeMeasure: LongitudeMeasure
    __sourceMapScale: SourceMapScale
    __horizontalAccuracyMeasure: MeasureCompact
    __verticalAccuracyMeasure: MeasureCompact
    __horizontalCollectionMethodName: HorizontalCollectionMethodName
    __horizontalCoordinateReferenceSystemDatumName: HorizontalCoordinateReferenceSystemDatumName
    __verticalMeasure: MeasureCompact
    __verticalCollectionMethodName: VerticalCollectionMethodName
    __verticalCoordinateReferenceSystemDatumName: VerticalCoordinateReferenceSystemDatumName
    __countryCode: CountryCode
    __stateCode: StateCode
    __countyCode: CountyCode

    def __init__(
        self,
        o: dict = None,
        *,
        latitudeMeasure: LatitudeMeasure = None,
        longitudeMeasure: LongitudeMeasure = None,
        sourceMapScale: SourceMapScale = None,
        horizontalAccuracyMeasure: MeasureCompact = None,
        verticalAccuracyMeasure: MeasureCompact = None,
        horizontalCollectionMethodName: HorizontalCollectionMethodName = None,
        horizontalCoordinateReferenceSystemDatumName: HorizontalCoordinateReferenceSystemDatumName = None,  # noqa: B950
        verticalMeasure: MeasureCompact = None,
        verticalCollectionMethodName: VerticalCollectionMethodName = None,
        verticalCoordinateReferenceSystemDatumName: VerticalCoordinateReferenceSystemDatumName = None,  # noqa: B950
        countryCode: CountryCode = None,
        stateCode: StateCode = None,
        countyCode: CountyCode = None
    ):
        if isinstance(o, MonitoringLocationGeospatial):
            # Assign attributes from object without typechecking
            self.__latitudeMeasure = o.latitudeMeasure
            self.__longitudeMeasure = o.longitudeMeasure
            self.__sourceMapScale = o.sourceMapScale
            self.__horizontalAccuracyMeasure = o.horizontalAccuracyMeasure
            self.__verticalAccuracyMeasure = o.verticalAccuracyMeasure
            self.__horizontalCollectionMethodName = o.horizontalCollectionMethodName
            self.__horizontalCoordinateReferenceSystemDatumName = (
                o.horizontalCoordinateReferenceSystemDatumName
            )
            self.__verticalMeasure = o.verticalMeasure
            self.__verticalCollectionMethodName = o.verticalCollectionMethodName
            self.__verticalCoordinateReferenceSystemDatumName = (
                o.verticalCoordinateReferenceSystemDatumName
            )
            self.__countryCode = o.countryCode
            self.__stateCode = o.stateCode
            self.__countyCode = o.countyCode
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.latitudeMeasure = o.get("latitudeMeasure")
            self.longitudeMeasure = o.get("longitudeMeasure")
            self.sourceMapScale = o.get("sourceMapScale")
            self.horizontalAccuracyMeasure = o.get("horizontalAccuracyMeasure")
            self.verticalAccuracyMeasure = o.get("verticalAccuracyMeasure")
            self.horizontalCollectionMethodName = o.get("horizontalCollectionMethodName")
            self.horizontalCoordinateReferenceSystemDatumName = o.get(
                "horizontalCoordinateReferenceSystemDatumName"
            )
            self.verticalMeasure = o.get("verticalMeasure")
            self.verticalCollectionMethodName = o.get("verticalCollectionMethodName")
            self.verticalCoordinateReferenceSystemDatumName = o.get(
                "verticalCoordinateReferenceSystemDatumName"
            )
            self.countryCode = o.get("countryCode")
            self.stateCode = o.get("stateCode")
            self.countyCode = o.get("countyCode")
        else:
            # Assign attributes from named keywords with typechecking
            self.latitudeMeasure = latitudeMeasure
            self.longitudeMeasure = longitudeMeasure
            self.sourceMapScale = sourceMapScale
            self.horizontalAccuracyMeasure = horizontalAccuracyMeasure
            self.verticalAccuracyMeasure = verticalAccuracyMeasure
            self.horizontalCollectionMethodName = horizontalCollectionMethodName
            self.horizontalCoordinateReferenceSystemDatumName = (
                horizontalCoordinateReferenceSystemDatumName
            )
            self.verticalMeasure = verticalMeasure
            self.verticalCollectionMethodName = verticalCollectionMethodName
            self.verticalCoordinateReferenceSystemDatumName = (
                verticalCoordinateReferenceSystemDatumName
            )
            self.countryCode = countryCode
            self.stateCode = stateCode
            self.countyCode = countyCode

    @property
    def latitudeMeasure(self) -> LatitudeMeasure:
        return self.__latitudeMeasure

    @latitudeMeasure.setter
    def latitudeMeasure(self, val: LatitudeMeasure) -> None:
        self.__latitudeMeasure = None if val is None else LatitudeMeasure(val)

    @property
    def longitudeMeasure(self) -> LongitudeMeasure:
        return self.__longitudeMeasure

    @longitudeMeasure.setter
    def longitudeMeasure(self, val: LongitudeMeasure) -> None:
        self.__longitudeMeasure = None if val is None else LongitudeMeasure(val)

    @property
    def sourceMapScale(self) -> SourceMapScale:
        return self.__sourceMapScale

    @sourceMapScale.setter
    def sourceMapScale(self, val: SourceMapScale) -> None:
        self.__sourceMapScale = None if val is None else SourceMapScale(val)

    @property
    def horizontalAccuracyMeasure(self) -> MeasureCompact:
        """
        The horizontal measure of the relative accuracy of the latitude and longitude
        coordinates.
        """
        return self.__horizontalAccuracyMeasure

    @horizontalAccuracyMeasure.setter
    def horizontalAccuracyMeasure(self, val: MeasureCompact) -> None:
        """
        The horizontal measure of the relative accuracy of the latitude and longitude
        coordinates.
        """
        self.__horizontalAccuracyMeasure = None if val is None else MeasureCompact(val)

    @property
    def verticalAccuracyMeasure(self) -> MeasureCompact:
        """
        Depth below land surface datum (LSD) to the bottom of the hole on completion of
        drilling.
        """
        return self.__verticalAccuracyMeasure

    @verticalAccuracyMeasure.setter
    def verticalAccuracyMeasure(self, val: MeasureCompact) -> None:
        """
        Depth below land surface datum (LSD) to the bottom of the hole on completion of
        drilling.
        """
        self.__verticalAccuracyMeasure = None if val is None else MeasureCompact(val)

    @property
    def horizontalCollectionMethodName(self) -> HorizontalCollectionMethodName:
        return self.__horizontalCollectionMethodName

    @horizontalCollectionMethodName.setter
    def horizontalCollectionMethodName(self, val: HorizontalCollectionMethodName) -> None:
        self.__horizontalCollectionMethodName = (
            None if val is None else HorizontalCollectionMethodName(val)
        )

    @property
    def horizontalCoordinateReferenceSystemDatumName(
        self,
    ) -> HorizontalCoordinateReferenceSystemDatumName:
        return self.__horizontalCoordinateReferenceSystemDatumName

    @horizontalCoordinateReferenceSystemDatumName.setter
    def horizontalCoordinateReferenceSystemDatumName(
        self, val: HorizontalCoordinateReferenceSystemDatumName
    ) -> None:
        self.__horizontalCoordinateReferenceSystemDatumName = (
            None if val is None else HorizontalCoordinateReferenceSystemDatumName(val)
        )

    @property
    def verticalMeasure(self) -> MeasureCompact:
        """
        The measure of elevation (i.e., the altitude), above or below a reference datum.
        """
        return self.__verticalMeasure

    @verticalMeasure.setter
    def verticalMeasure(self, val: MeasureCompact) -> None:
        """
        The measure of elevation (i.e., the altitude), above or below a reference datum.
        """
        self.__verticalMeasure = None if val is None else MeasureCompact(val)

    @property
    def verticalCollectionMethodName(self) -> VerticalCollectionMethodName:
        return self.__verticalCollectionMethodName

    @verticalCollectionMethodName.setter
    def verticalCollectionMethodName(self, val: VerticalCollectionMethodName) -> None:
        self.__verticalCollectionMethodName = (
            None if val is None else VerticalCollectionMethodName(val)
        )

    @property
    def verticalCoordinateReferenceSystemDatumName(
        self,
    ) -> VerticalCoordinateReferenceSystemDatumName:
        return self.__verticalCoordinateReferenceSystemDatumName

    @verticalCoordinateReferenceSystemDatumName.setter
    def verticalCoordinateReferenceSystemDatumName(
        self, val: VerticalCoordinateReferenceSystemDatumName
    ) -> None:
        self.__verticalCoordinateReferenceSystemDatumName = (
            None if val is None else VerticalCoordinateReferenceSystemDatumName(val)
        )

    @property
    def countryCode(self) -> CountryCode:
        return self.__countryCode

    @countryCode.setter
    def countryCode(self, val: CountryCode) -> None:
        self.__countryCode = None if val is None else CountryCode(val)

    @property
    def stateCode(self) -> StateCode:
        return self.__stateCode

    @stateCode.setter
    def stateCode(self, val: StateCode) -> None:
        self.__stateCode = None if val is None else StateCode(val)

    @property
    def countyCode(self) -> CountyCode:
        return self.__countyCode

    @countyCode.setter
    def countyCode(self, val: CountyCode) -> None:
        self.__countyCode = None if val is None else CountyCode(val)

    def generateXML(  # noqa: C901
        self, name: str = "MonitoringLocationGeospatial"
    ) -> str:
        doc = Doc()
        asis = doc.asis
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__latitudeMeasure is None:
                raise WQXException("Attribute 'LatitudeMeasure' is required.")
            line("LatitudeMeasure", str(self.__latitudeMeasure))
            if self.__longitudeMeasure is None:
                raise WQXException("Attribute 'LongitudeMeasure' is required.")
            line("LongitudeMeasure", str(self.__longitudeMeasure))
            if self.__sourceMapScale is not None:
                line("SourceMapScale", self.__sourceMapScale)
            if self.__horizontalAccuracyMeasure is not None:
                asis(
                    self.__horizontalAccuracyMeasure.generateXML(
                        "HorizontalAccuracyMeasure"
                    )
                )
            if self.__verticalAccuracyMeasure is not None:
                asis(
                    self.__verticalAccuracyMeasure.generateXML("VerticalAccuracyMeasure")
                )
            if self.__horizontalCollectionMethodName is None:
                raise WQXException(
                    "Attribute 'HorizontalCollectionMethodName' is required."
                )
            line("HorizontalCollectionMethodName", self.__horizontalCollectionMethodName)
            if self.__horizontalCoordinateReferenceSystemDatumName is None:
                raise WQXException(
                    "Attribute 'HorizontalCoordinateReferenceSystemDatumName' is required."
                )
            line(
                "HorizontalCoordinateReferenceSystemDatumName",
                self.__horizontalCoordinateReferenceSystemDatumName,
            )
            if self.__verticalMeasure is not None:
                asis(self.__verticalMeasure.generateXML("VerticalMeasure"))
            if self.__verticalCollectionMethodName is not None:
                line("VerticalCollectionMethodName", self.__verticalCollectionMethodName)
            if self.__verticalCoordinateReferenceSystemDatumName is not None:
                line(
                    "VerticalCoordinateReferenceSystemDatumName",
                    self.__verticalCoordinateReferenceSystemDatumName,
                )
            if self.__countryCode is not None:
                line("CountryCode", self.__countryCode)
            if self.__stateCode is not None:
                line("StateCode", self.__stateCode)
            if self.__countyCode is not None:
                line("CountyCode", self.__countyCode)

        return doc.getvalue()



from yattag import Doc

from ..exceptions import WQXException
from .MeasureCompact import MeasureCompact
from .SimpleContent import (
    DetectionQuantitationLimitCommentText,
    DetectionQuantitationLimitTypeName,
)


class DetectionQuantitationLimit:
    """
    Information that describes one of a variety of detection or quantitation limits
    determined in a laboratory.
    """

    __detectionQuantitationLimitTypeName: DetectionQuantitationLimitTypeName
    __detectionQuantitationLimitMeasure: MeasureCompact
    __detectionQuantitationLimitCommentText: DetectionQuantitationLimitCommentText

    def __init__(
        self,
        o: dict = None,
        *,
        detectionQuantitationLimitTypeName: DetectionQuantitationLimitTypeName = None,
        detectionQuantitationLimitMeasure: MeasureCompact = None,
        detectionQuantitationLimitCommentText: DetectionQuantitationLimitCommentText = None
    ):
        if isinstance(o, DetectionQuantitationLimit):
            # Assign attributes from object without typechecking
            self.__detectionQuantitationLimitTypeName = (
                o.detectionQuantitationLimitTypeName
            )
            self.__detectionQuantitationLimitMeasure = o.detectionQuantitationLimitMeasure
            self.__detectionQuantitationLimitCommentText = (
                o.detectionQuantitationLimitCommentText
            )
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.detectionQuantitationLimitTypeName = o.get(
                "detectionQuantitationLimitTypeName"
            )
            self.detectionQuantitationLimitMeasure = o.get(
                "detectionQuantitationLimitMeasure"
            )
            self.detectionQuantitationLimitCommentText = o.get(
                "detectionQuantitationLimitCommentText"
            )
        else:
            # Assign attributes from named keywords with typechecking
            self.detectionQuantitationLimitTypeName = detectionQuantitationLimitTypeName
            self.detectionQuantitationLimitMeasure = detectionQuantitationLimitMeasure
            self.detectionQuantitationLimitCommentText = (
                detectionQuantitationLimitCommentText
            )

    @property
    def detectionQuantitationLimitTypeName(self) -> DetectionQuantitationLimitTypeName:
        return self.__detectionQuantitationLimitTypeName

    @detectionQuantitationLimitTypeName.setter
    def detectionQuantitationLimitTypeName(
        self, val: DetectionQuantitationLimitTypeName
    ) -> None:
        self.__detectionQuantitationLimitTypeName = DetectionQuantitationLimitTypeName(
            val
        )

    @property
    def detectionQuantitationLimitMeasure(self) -> MeasureCompact:
        return self.__detectionQuantitationLimitMeasure

    @detectionQuantitationLimitMeasure.setter
    def detectionQuantitationLimitMeasure(self, val: MeasureCompact) -> None:
        self.__detectionQuantitationLimitMeasure = MeasureCompact(val)

    @property
    def detectionQuantitationLimitCommentText(
        self,
    ) -> DetectionQuantitationLimitCommentText:
        return self.__detectionQuantitationLimitCommentText

    @detectionQuantitationLimitCommentText.setter
    def detectionQuantitationLimitCommentText(
        self, val: DetectionQuantitationLimitCommentText
    ) -> None:
        self.__detectionQuantitationLimitCommentText = (
            None if val is None else DetectionQuantitationLimitCommentText(val)
        )

    def generateXML(self, name: str = "DetectionQuantitationLimit") -> str:
        doc = Doc()
        asis = doc.asis
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__detectionQuantitationLimitTypeName is None:
                raise WQXException(
                    "Attribute 'detectionQuantitationLimitTypeName' is required."
                )
            line(
                "DetectionQuantitationLimitTypeName",
                self.__detectionQuantitationLimitTypeName,
            )
            if self.__detectionQuantitationLimitMeasure is None:
                raise WQXException(
                    "Attribute 'detectionQuantitationLimitMeasure' is required."
                )
            asis(
                self.__detectionQuantitationLimitMeasure.generateXML(
                    "DetectionQuantitationLimitMeasure"
                )
            )
            if self.__detectionQuantitationLimitCommentText is not None:
                line(
                    "DetectionQuantitationLimitCommentText",
                    self.__detectionQuantitationLimitCommentText,
                )

        return doc.getvalue()



from typing import List, Union

from yattag import Doc

from ..exceptions import WQXException
from .AttachedBinaryObject import AttachedBinaryObject
from .BiologicalResultDescription import BiologicalResultDescription
from .ComparableAnalyticalMethod import ComparableAnalyticalMethod
from .LabSamplePreparation import LabSamplePreparation
from .ResultAnalyticalMethod import ResultAnalyticalMethod
from .ResultDescription import ResultDescription
from .ResultLabInformation import ResultLabInformation


class Result:
    """
    Describes the results of a field measurement, observation, or laboratory analysis.
    """

    __resultDescription: ResultDescription
    __biologicalResultDescription: BiologicalResultDescription
    __attachedBinaryObject: List[AttachedBinaryObject]
    __resultAnalyticalMethod: ResultAnalyticalMethod
    __comparableAnalyticalMethod: ComparableAnalyticalMethod
    __resultLabInformation: ResultLabInformation
    __labSamplePreparation: List[LabSamplePreparation]

    def __init__(
        self,
        o: dict = None,
        *,
        resultDescription: ResultDescription = None,
        biologicalResultDescription: BiologicalResultDescription = None,
        attachedBinaryObject: List[AttachedBinaryObject] = None,
        resultAnalyticalMethod: ResultAnalyticalMethod = None,
        comparableAnalyticalMethod: ComparableAnalyticalMethod = None,
        resultLabInformation: ResultLabInformation = None,
        labSamplePreparation: List[LabSamplePreparation] = None
    ):
        if isinstance(o, Result):
            # Assign attributes from object without typechecking
            self.__resultDescription = o.resultDescription
            self.__biologicalResultDescription = o.biologicalResultDescription
            self.__attachedBinaryObject = o.attachedBinaryObject
            self.__resultAnalyticalMethod = o.resultAnalyticalMethod
            self.__comparableAnalyticalMethod = o.comparableAnalyticalMethod
            self.__resultLabInformation = o.resultLabInformation
            self.__labSamplePreparation = o.labSamplePreparation
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.resultDescription = o.get("resultDescription")
            self.biologicalResultDescription = o.get("biologicalResultDescription")
            self.attachedBinaryObject = o.get("attachedBinaryObject")
            self.resultAnalyticalMethod = o.get("resultAnalyticalMethod")
            self.comparableAnalyticalMethod = o.get("comparableAnalyticalMethod")
            self.resultLabInformation = o.get("resultLabInformation")
            self.labSamplePreparation = o.get("labSamplePreparation")
        else:
            # Assign attributes from named keywords with typechecking
            self.resultDescription = resultDescription
            self.biologicalResultDescription = biologicalResultDescription
            self.attachedBinaryObject = attachedBinaryObject
            self.resultAnalyticalMethod = resultAnalyticalMethod
            self.comparableAnalyticalMethod = comparableAnalyticalMethod
            self.resultLabInformation = resultLabInformation
            self.labSamplePreparation = labSamplePreparation

    @property
    def resultDescription(self) -> ResultDescription:
        return self.__resultDescription

    @resultDescription.setter
    def resultDescription(self, val: ResultDescription) -> None:
        self.__resultDescription = None if val is None else ResultDescription(val)

    @property
    def biologicalResultDescription(self) -> BiologicalResultDescription:
        return self.__biologicalResultDescription

    @biologicalResultDescription.setter
    def biologicalResultDescription(self, val: BiologicalResultDescription) -> None:
        self.__biologicalResultDescription = (
            None if val is None else BiologicalResultDescription(val)
        )

    @property
    def attachedBinaryObject(self) -> List[AttachedBinaryObject]:
        return self.__attachedBinaryObject

    @attachedBinaryObject.setter
    def attachedBinaryObject(
        self, val: Union[AttachedBinaryObject, List[AttachedBinaryObject]]
    ) -> None:
        if val is None:
            self.__attachedBinaryObject = []
        elif isinstance(val, list):
            r: List[AttachedBinaryObject] = []
            for x in val:
                r.append(AttachedBinaryObject(x))
            self.__attachedBinaryObject = r
        else:
            self.__attachedBinaryObject = [AttachedBinaryObject(val)]

    @property
    def resultAnalyticalMethod(self) -> ResultAnalyticalMethod:
        return self.__resultAnalyticalMethod

    @resultAnalyticalMethod.setter
    def resultAnalyticalMethod(self, val: ResultAnalyticalMethod) -> None:
        self.__resultAnalyticalMethod = (
            None if val is None else ResultAnalyticalMethod(val)
        )

    @property
    def comparableAnalyticalMethod(self) -> ComparableAnalyticalMethod:
        return self.__comparableAnalyticalMethod

    @comparableAnalyticalMethod.setter
    def comparableAnalyticalMethod(self, val: ComparableAnalyticalMethod) -> None:
        self.__comparableAnalyticalMethod = (
            None if val is None else ComparableAnalyticalMethod(val)
        )

    @property
    def resultLabInformation(self) -> ResultLabInformation:
        return self.__resultLabInformation

    @resultLabInformation.setter
    def resultLabInformation(self, val: ResultLabInformation) -> None:
        self.__resultLabInformation = None if val is None else ResultLabInformation(val)

    @property
    def labSamplePreparation(self) -> List[LabSamplePreparation]:
        return self.__labSamplePreparation

    @labSamplePreparation.setter
    def labSamplePreparation(
        self, val: Union[LabSamplePreparation, List[LabSamplePreparation]]
    ) -> None:
        if val is None:
            self.__labSamplePreparation = []
        elif isinstance(val, list):
            r: List[LabSamplePreparation] = []
            for x in val:
                r.append(LabSamplePreparation(x))
            self.__labSamplePreparation = r
        else:
            self.__labSamplePreparation = [LabSamplePreparation(val)]

    def generateXML(self, name: str = "Result") -> str:
        doc = Doc()
        asis = doc.asis
        tag = doc.tag

        with tag(name):
            if self.__resultDescription is None:
                raise WQXException("Attribute 'resultDescription' is required.")
            asis(self.__resultDescription.generateXML("ResultDescription"))
            if self.__biologicalResultDescription is not None:
                asis(
                    self.__biologicalResultDescription.generateXML(
                        "BiologicalResultDescription"
                    )
                )
            for x in self.__attachedBinaryObject:
                asis(x.generateXML("AttachedBinaryObject"))
            if self.__resultAnalyticalMethod is not None:
                asis(self.__resultAnalyticalMethod.generateXML("ResultAnalyticalMethod"))
            if self.__comparableAnalyticalMethod is not None:
                asis(
                    self.__comparableAnalyticalMethod.generateXML(
                        "ComparableAnalyticalMethod"
                    )
                )
            if self.__resultLabInformation is not None:
                asis(self.__resultLabInformation.generateXML("ResultLabInformation"))
            for x in self.__labSamplePreparation:
                asis(x.generateXML("LabSamplePreparation"))

        return doc.getvalue()



from .Activity import *  # noqa: F401,F403
from .ActivityDescription import *  # noqa: F401,F403
from .ActivityGroup import *  # noqa: F401,F403
from .ActivityLocation import *  # noqa: F401,F403
from .ActivityMetric import *  # noqa: F401,F403
from .ActivityMetricType import *  # noqa: F401,F403
from .AlternateMonitoringLocationIdentity import *  # noqa: F401,F403
from .AquiferInformation import *  # noqa: F401,F403
from .AttachedBinaryObject import *  # noqa: F401,F403
from .BibliographicReference import *  # noqa: F401,F403
from .BiologicalActivityDescription import *  # noqa: F401,F403
from .BiologicalHabitatCollectionInformation import *  # noqa: F401,F403
from .BiologicalHabitatIndex import *  # noqa: F401,F403
from .CollectionEffort import *  # noqa: F401,F403
from .ComparableAnalyticalMethod import *  # noqa: F401,F403
from .DataQualityIndicator import *  # noqa: F401,F403
from .DetectionQuantitationLimit import *  # noqa: F401,F403
from .ElectronicAddress import *  # noqa: F401,F403
from .Entity_Update_Identifiers import *  # noqa: F401,F403
from .FrequencyClassInformation import *  # noqa: F401,F403
from .HorizontalAccuracyMeasure import *  # noqa: F401,F403
from .IndexType import *  # noqa: F401,F403
from .LabSamplePreparation import *  # noqa: F401,F403
from .Measure import *  # noqa: F401,F403
from .MeasureCompact import *  # noqa: F401,F403
from .MonitoringLocation import *  # noqa: F401,F403
from .MonitoringLocationGeospatial import *  # noqa: F401,F403
from .MonitoringLocationIdentity import *  # noqa: F401,F403
from .NetInformation import *  # noqa: F401,F403
from .Organization import *  # noqa: F401,F403
from .Organization_Delete import *  # noqa: F401,F403
from .OrganizationAddress import *  # noqa: F401,F403
from .OrganizationDescription import *  # noqa: F401,F403
from .Project import *  # noqa: F401,F403
from .ProjectMonitoringLocationWeighting import *  # noqa: F401,F403
from .ReferenceMethod import *  # noqa: F401,F403
from .Result import *  # noqa: F401,F403
from .ResultAnalyticalMethod import *  # noqa: F401,F403
from .ResultDescription import *  # noqa: F401,F403
from .ResultLabInformation import *  # noqa: F401,F403
from .SampleDescription import *  # noqa: F401,F403
from .SamplePreparation import *  # noqa: F401,F403
from .SimpleContent import *  # noqa: F401,F403
from .TaxonomicDetails import *  # noqa: F401,F403
from .Telephonic import *  # noqa: F401,F403
from .WellInformation import *  # noqa: F401,F403
from .WQX import *  # noqa: F401,F403
from .WQX_Delete import *  # noqa: F401,F403
from .WQX_Update_Identifiers import *  # noqa: F401,F403
from .WQXTime import *  # noqa: F401,F403



from typing import List, Union

from yattag import Doc

from ..exceptions import WQXException
from .Organization_Delete import OrganizationDelete


class WQXDelete:
    """
    Main Schema used to delete a portion of water monitoring results from EPA Office of
    Water system.
    """

    __organizationDelete: List[OrganizationDelete]

    def __init__(
        self, o: dict = None, *, organizationDelete: List[OrganizationDelete] = None
    ):
        if isinstance(o, WQXDelete):
            # Assign attributes from object without typechecking
            self.__organizationDelete = o.organizationDelete
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.organizationDelete = o.get("organizationDelete")
        else:
            # Assign attributes from named keywords with typechecking
            self.organizationDelete = organizationDelete

    @property
    def organizationDelete(self) -> List[OrganizationDelete]:
        return self.__organizationDelete

    @organizationDelete.setter
    def organizationDelete(
        self, val: Union[OrganizationDelete, List[OrganizationDelete]]
    ) -> None:
        if val is None:
            self.__organizationDelete = []
        elif isinstance(val, list):
            r: List[OrganizationDelete] = []
            for x in val:
                r.append(OrganizationDelete(x))
            self.__organizationDelete = r
        else:
            self.__organizationDelete = [OrganizationDelete(val)]

    def generateXML(self, name="WQXDelete"):
        doc = Doc()
        asis = doc.asis
        tag = doc.tag

        with tag(
            name,
            ("xmlns", "http://www.exchangenetwork.net/schema/wqx/3"),
            ("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance"),
            (
                "xsi:schemaLocation",
                "http://www.exchangenetwork.net/schema/wqx/3 "
                "http://www.exchangenetwork.net/schema/wqx/3/index.xsd",
            ),
        ):
            if self.__organizationDelete is None:
                raise WQXException("Attribute 'organizationDelete' is required.")
            for x in self.__organizationDelete:
                asis(x.generateXML("OrganizationDelete"))

        return doc.getvalue()
